@@ -1,7 +1,7 @@
 /* capture.c
  * Routines for packet capture windows
  *
- * $Id: capture.c,v 1.254 2004/06/30 06:58:56 guy Exp $
+ * $Id: capture.c 12115 2004-09-27 22:55:15Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -140,7 +140,7 @@
 #include "pcap-util.h"
 #include "alert_box.h"
 #include "simple_dialog.h"
-#include "prefs.h"
+#include <epan/prefs.h>
 #include "globals.h"
 #include "conditions.h"
 #include "capture_stop_conditions.h"
@@ -150,22 +150,22 @@
 #include "wiretap/wtap.h"
 #include "wiretap/wtap-capture.h"
 
-#include "packet-ap1394.h"
-#include "packet-atalk.h"
-#include "packet-atm.h"
-#include "packet-clip.h"
-#include "packet-eth.h"
-#include "packet-fddi.h"
-#include "packet-null.h"
-#include "packet-ppp.h"
-#include "packet-raw.h"
-#include "packet-sll.h"
-#include "packet-tr.h"
-#include "packet-ieee80211.h"
-#include "packet-chdlc.h"
-#include "packet-prism.h"
-#include "packet-ipfc.h"
-#include "packet-arcnet.h"
+#include <epan/dissectors/packet-ap1394.h>
+#include <epan/dissectors/packet-atalk.h>
+#include <epan/dissectors/packet-atm.h>
+#include <epan/dissectors/packet-clip.h>
+#include <epan/dissectors/packet-eth.h>
+#include <epan/dissectors/packet-fddi.h>
+#include <epan/dissectors/packet-null.h>
+#include <epan/dissectors/packet-ppp.h>
+#include <epan/dissectors/packet-raw.h>
+#include <epan/dissectors/packet-sll.h>
+#include <epan/dissectors/packet-tr.h>
+#include <epan/dissectors/packet-ieee80211.h>
+#include <epan/dissectors/packet-chdlc.h>
+#include <epan/dissectors/packet-prism.h>
+#include <epan/dissectors/packet-ipfc.h>
+#include <epan/dissectors/packet-arcnet.h>
 
 #ifdef _WIN32
 #include "capture-wpcap.h"
@@ -420,7 +420,12 @@ sync_pipe_do_capture(gboolean is_tempfile) {
 
     if (capture_opts.linktype != -1) {
       argv = sync_pipe_add_arg(argv, &argc, "-y");
+#ifdef HAVE_PCAP_DATALINK_VAL_TO_NAME
+      sprintf(ssnap,"%s",pcap_datalink_val_to_name(capture_opts.linktype));
+#else
+      /* XXX - just treat it as a number */
       sprintf(ssnap,"%d",capture_opts.linktype);
+#endif
       argv = sync_pipe_add_arg(argv, &argc, ssnap);
     }
 
