@@ -1,7 +1,7 @@
 /* packet.c
  * Routines for packet disassembly
  *
- * $Id: packet.c,v 1.103 2004/02/28 20:59:23 guy Exp $
+ * $Id: packet.c 11894 2004-09-04 11:35:26Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -48,7 +48,7 @@
 #include "osi-utils.h"
 #include "to_str.h"
 
-#include "resolv.h"
+#include "addr_resolv.h"
 #include "tvbuff.h"
 #include "plugins.h"
 #include "epan_dissect.h"
@@ -189,7 +189,7 @@ add_new_data_source(packet_info *pinfo, tvbuff_t *tvb, char *name)
 	 * we don't need to allocate a buffer for a copy and make a
 	 * copy, and wouldn't need to free the buffer, either.
 	 */
-	src->name = g_strdup(name);
+	src->name = g_strdup_printf("%s (%u bytes)", name, tvb_length(tvb));
 	pinfo->data_src = g_slist_append(pinfo->data_src, src);
 }
 
@@ -297,6 +297,8 @@ dissect_packet(epan_dissect_t *edt, union wtap_pseudo_header *pseudo_header,
         edt->pi.src_idx = 0;
         edt->pi.dst_idx = 0;
         edt->pi.vsan = 0;
+        edt->pi.dcectxid = 0;
+        edt->pi.dcetransporttype = -1;
 
 	TRY {
 		edt->tvb = tvb_new_real_data(pd, fd->cap_len, fd->pkt_len);
