@@ -1,7 +1,7 @@
 /* proto.h
  * Definitions for protocol display
  *
- * $Id: proto.h,v 1.68 2004/05/15 00:52:02 ulfl Exp $
+ * $Id: proto.h 11954 2004-09-10 15:21:20Z gram $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -50,8 +50,8 @@
 #include "tvbuff.h"
 #include "ftypes/ftypes.h"
 
-/** The header-field index for the special text pseudo-field */
-extern int hf_text_only;
+/** The header-field index for the special text pseudo-field. Exported by libethereal.dll */
+ETH_VAR_IMPORT int hf_text_only;
 
 /** the maximum length of a protocol field string representation */
 #define ITEM_LABEL_LENGTH	240
@@ -747,6 +747,32 @@ extern proto_item *
 proto_tree_add_uint_format(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start,
 	gint length, guint32 value, const char *format, ...) GNUC_FORMAT_CHECK(printf,7,8);
 
+/** Add an FT_UINT64 to a proto_tree.
+ @param tree the tree to append this item to
+ @param hfindex field index
+ @param tvb the tv buffer of the current data
+ @param start start of data in tvb
+ @param length length of data in tvb
+ @param value data to display
+ @return the newly created item */
+extern proto_item *
+proto_tree_add_uint64(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start,
+	gint length, guint64 value);
+
+/** Add a formatted FT_UINT64 to a proto_tree.
+ @param tree the tree to append this item to
+ @param hfindex field index
+ @param tvb the tv buffer of the current data
+ @param start start of data in tvb
+ @param length length of data in tvb
+ @param value data to display
+ @param format printf like format string
+ @param ... printf like parameters
+ @return the newly created item */
+extern proto_item *
+proto_tree_add_uint64_format(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start,
+	gint length, guint64 value, const char *format, ...) GNUC_FORMAT_CHECK(printf,7,8);
+
 /** Add one of FT_INT8, FT_INT16, FT_INT24 or FT_INT32 to a proto_tree.
  @param tree the tree to append this item to
  @param hfindex field index
@@ -779,6 +805,32 @@ extern proto_item *
 proto_tree_add_int_format(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start,
 	gint length, gint32 value, const char *format, ...) GNUC_FORMAT_CHECK(printf,7,8);
 
+/** Add an FT_INT64 to a proto_tree.
+ @param tree the tree to append this item to
+ @param hfindex field index
+ @param tvb the tv buffer of the current data
+ @param start start of data in tvb
+ @param length length of data in tvb
+ @param value data to display
+ @return the newly created item */
+extern proto_item *
+proto_tree_add_int64(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start,
+	gint length, gint64 value);
+
+/** Add a formatted FT_INT64 to a proto_tree.
+ @param tree the tree to append this item to
+ @param hfindex field index
+ @param tvb the tv buffer of the current data
+ @param start start of data in tvb
+ @param length length of data in tvb
+ @param value data to display
+ @param format printf like format string
+ @param ... printf like parameters
+ @return the newly created item */
+extern proto_item *
+proto_tree_add_int64_format(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start,
+	gint length, gint64 value, const char *format, ...) GNUC_FORMAT_CHECK(printf,7,8);
+
 /** Useful for quick debugging. Also sends string to STDOUT, so don't
  * leave call to this function in production code.
  @param tree the tree to append the text to
@@ -946,8 +998,13 @@ extern GPtrArray* proto_find_finfo(proto_tree *tree, int hfindex);
 /** Dumps a glossary of the protocol registrations to STDOUT */
 extern void proto_registrar_dump_protocols(void);
 
-/** Dumps a glossary of the protocol and field registrations to STDOUT */
-extern void proto_registrar_dump_fields(void);
+/** Dumps a glossary of the field value strings or true/false strings to STDOUT */
+extern void proto_registrar_dump_values(void);
+
+/** Dumps a glossary of the protocol and field registrations to STDOUT.
+ * Format 1 is the original format. Format 2 includes the base (for integers)
+ * and the blurb. */
+extern void proto_registrar_dump_fields(int format);
 
 
 
