@@ -1,7 +1,7 @@
 /* etypes.h
  * Defines ethernet packet types, similar to tcpdump's ethertype.h
  *
- * $Id: etypes.h,v 1.38 2004/02/28 22:56:35 guy Exp $
+ * $Id: etypes.h 11536 2004-07-27 06:05:20Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -64,6 +64,10 @@
 #define ETHERTYPE_CGMP		0x2001
 #endif
 
+#ifndef ETHERTYPE_CENTRINO_PROMISC
+#define ETHERTYPE_CENTRINO_PROMISC	0x2452	/* Intel Centrino promiscuous packets */
+#endif
+
 #ifndef ETHERTYPE_3C_NBP_DGRAM
 #define ETHERTYPE_3C_NBP_DGRAM	0x3c07
 #endif
@@ -164,8 +168,12 @@
 #define ETHERTYPE_IPv6		0x86dd
 #endif
 
+#ifndef ETHERTYPE_MAC_CONTROL
+#define ETHERTYPE_MAC_CONTROL	0x8808
+#endif
+
 #ifndef ETHERTYPE_SLOW_PROTOCOLS
-#define ETHERTYPE_SLOW_PROTOCOLS		0x8809
+#define ETHERTYPE_SLOW_PROTOCOLS	0x8809
 #endif
 
 #ifndef ETHERTYPE_PPP
@@ -212,6 +220,10 @@
 #define ETHERTYPE_BRDWALK       0x88AE
 #endif
 
+#ifndef ETHERTYPE_RSN_PREAUTH
+#define ETHERTYPE_RSN_PREAUTH	0x88c7  /* 802.11i Pre-Authentication */
+#endif
+
 #ifndef ETHERTYPE_LOOP
 #define ETHERTYPE_LOOP		0x9000 	/* used for layer 2 testing (do i see my own frames on the wire) */
 #endif
