@@ -1,7 +1,7 @@
 /* capture_dlg.c
  * Routines for packet capture windows
  *
- * $Id: capture_dlg.c,v 1.137 2004/06/30 18:24:56 ulfl Exp $
+ * $Id: capture_dlg.c 12272 2004-10-12 06:11:30Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,22 +35,23 @@
 #include <epan/packet.h>
 #include "capture.h"
 #include "globals.h"
-#include <epan/resolv.h>
+#include <epan/addr_resolv.h>
 #include "main.h"
 #include "ui_util.h"
 #include "capture_dlg.h"
-#include "filter_prefs.h"
+#include "filter_dlg.h"
 #include "simple_dialog.h"
 #include "dlg_utils.h"
 #include "pcap-util.h"
 #include "capture_combo_utils.h"
-#include "prefs.h"
+#include <epan/prefs.h>
 #include "ringbuffer.h"
 #include <epan/filesystem.h>
 #include "compat_macros.h"
 #include "file_dlg.h"
 #include "help_dlg.h"
 #include "gtkglobals.h"
+#include "cfilter_combo_utils.h"
 
 #ifdef _WIN32
 #include "capture-wpcap.h"
@@ -58,6 +59,7 @@
 
 /* Capture callback data keys */
 #define E_CAP_IFACE_KEY             "cap_iface"
+#define E_CAP_IFACE_IP_KEY          "cap_iface_ip"
 #define E_CAP_SNAP_CB_KEY           "cap_snap_cb"
 #define E_CAP_LT_OM_KEY             "cap_lt_om"
 #define E_CAP_LT_OM_LABEL_KEY       "cap_lt_om_label"
@@ -123,87 +125,6 @@ capture_stop_cb(GtkWidget *w _U_, gpointer d _U_)
     capture_stop();
 }
 
-/*
- * Given text that contains an interface name possibly prefixed by an
- * interface description, extract the interface name.
- */
-static char *
-get_if_name(char *if_text)
-{
-  char *if_name;
-
-#ifdef WIN32
-  /*
-   * We cannot assume that the interface name doesn't contain a space;
-   * some names on Windows OT do.
-   *
-   * We also can't assume it begins with "\Device\", either, as, on
-   * Windows OT, WinPcap doesn't put "\Device\" in front of the name.
-   *
-   * As I remember, we can't assume that the interface description
-   * doesn't contain a colon, either; I think some do.
-   *
-   * We can probably assume that the interface *name* doesn't contain
-   * a colon, however; if any interface name does contain a colon on
-   * Windows, it'll be time to just get rid of the damn interface
-   * descriptions in the drop-down list, have just the names in the
-   * drop-down list, and have a "Browse..." button to browse for interfaces,
-   * with names, descriptions, IP addresses, blah blah blah available when
-   * possible.
-   *
-   * So we search backwards for a colon.  If we don't find it, just
-   * return the entire string; otherwise, skip the colon and any blanks
-   * after it, and return that string.
-   */
-   if_name = if_text + strlen(if_text);
-   for (;;) {
-     if (if_name == if_text) {
-       /* We're at the beginning of the string; return it. */
-       break;
-     }
-     if_name--;
-     if (*if_name == ':') {
-       /*
-        * We've found a colon.
-        * Unfortunately, a colon is used in the string "rpcap://",
-        * which is used in case of a remote capture.
-        * So we'll check to make sure the colon isn't followed by "//";
-        * it'll be followed by a blank if it separates the description
-        * and the interface name.  (We don't wire in "rpcap", in case we
-        * support other protocols in the same syntax.)
-        */
-       if (strncmp(if_name, "://", 3) != 0) {
-         /*
-          * OK, we've found a colon not followed by "//".  Skip blanks
-          * following it.
-          */
-         if_name++;
-         while (*if_name == ' ')
-           if_name++;
-         break;
-       }
-     }
-     /* Keep looking for a colon not followed by "//". */
-   }
-#else
-  /*
-   * There's a space between the interface description and name, and
-   * the interface name shouldn't have a space in it (it doesn't, on
-   * UNIX systems); look backwards in the string for a space.
-   *
-   * (An interface name might, however, contain a colon in it, which
-   * is why we don't use the colon search on UNIX.)
-   */
-  if_name = strrchr(if_text, ' ');
-  if (if_name == NULL) {
-    if_name = if_text;
-  } else {
-    if_name++;
-  }
-#endif
-  return if_name;
-}
-
 /*
  * Keep a static pointer to the current "Capture Options" window, if
  * any, so that if somebody tries to do "Capture:Start" while there's
@@ -230,6 +151,11 @@ set_link_type_list(GtkWidget *linktype_om, GtkWidget *entry)
   gchar *linktype_menu_label;
   guint num_supported_link_types;
   GtkWidget *linktype_lb = OBJECT_GET_DATA(linktype_om, E_CAP_LT_OM_LABEL_KEY);
+  GtkWidget *if_ip_lb;
+  GString *ip_str = g_string_new("IP address: ");
+  int ips = 0;
+  GSList *curr_ip;
+  if_addr_t *ip_addr;
 
   lt_menu = gtk_menu_new();
   entry_text = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
@@ -267,6 +193,32 @@ set_link_type_list(GtkWidget *linktype_om, GtkWidget *entry)
 	   * Get the list of link-layer types for it.
 	   */
 	  lt_list = get_pcap_linktype_list(if_name, err_buf);
+
+	  /* create string of list of IP addresses of this interface */
+	  for (; (curr_ip = g_slist_nth(if_info->ip_addr, ips)) != NULL; ips++) {
+	    if (ips != 0)
+	      g_string_append(ip_str, ", ");
+
+	    ip_addr = (if_addr_t *)curr_ip->data;
+	    switch (ip_addr->type) {
+
+	    case AT_IPv4:
+	      g_string_append(ip_str,
+		ip_to_str((guint8 *)&ip_addr->ip_addr.ip4_addr));
+	      break;
+
+	    case AT_IPv6:
+	      g_string_append(ip_str,
+	          ip6_to_str((struct e_in6_addr *)&ip_addr->ip_addr.ip6_addr));
+	      break;
+
+            default:
+              g_assert_not_reached();
+	    }
+	  }
+
+	  if (if_info->loopback)
+	    g_string_append(ip_str, " (loopback)");
 	}
       }
       free_interface_list(if_list);
@@ -297,6 +249,13 @@ set_link_type_list(GtkWidget *linktype_om, GtkWidget *entry)
   gtk_option_menu_set_menu(GTK_OPTION_MENU(linktype_om), lt_menu);
   gtk_widget_set_sensitive(linktype_lb, num_supported_link_types >= 2);
   gtk_widget_set_sensitive(linktype_om, num_supported_link_types >= 2);
+
+  if_ip_lb = OBJECT_GET_DATA(linktype_om, E_CAP_IFACE_KEY);
+  if(ips == 0) {
+    g_string_append(ip_str, "unknown");
+  }
+  gtk_label_set_text(GTK_LABEL(if_ip_lb), ip_str->str);
+  g_string_free(ip_str, TRUE);
 }
 
 
@@ -324,7 +283,7 @@ static GtkWidget *time_unit_option_menu_new(guint32 value) {
 		gtk_menu_append(GTK_MENU(menu), menu_item);
 	}
 
-    /* the selected menu item can't be changed, once the option_menu 
+    /* the selected menu item can't be changed, once the option_menu
        is created, so set the matching menu item now */
     /* days */
     if(value >= 60 * 60 * 24) {
@@ -377,7 +336,7 @@ guint32 value)
 {
 	GtkWidget *menu, *menu_item;
     int unit;
-        
+
     menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(unit_om));
     menu_item = gtk_menu_get_active(GTK_MENU(menu));
     unit = GPOINTER_TO_INT(OBJECT_GET_DATA(menu_item, "time_unit"));
@@ -427,7 +386,7 @@ static GtkWidget *size_unit_option_menu_new(guint32 value) {
 		gtk_menu_append(GTK_MENU(menu), menu_item);
 	}
 
-    /* the selected menu item can't be changed, once the option_menu 
+    /* the selected menu item can't be changed, once the option_menu
        is created, so set the matching menu item now */
     /* gigabytes */
     if(value >= 1024 * 1024 * 1024) {
@@ -446,7 +405,7 @@ static GtkWidget *size_unit_option_menu_new(guint32 value) {
             }
         }
     }
-        
+
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(unit_om), menu);
 
     return unit_om;
@@ -480,7 +439,7 @@ guint32 value)
 {
 	GtkWidget *menu, *menu_item;
     int unit;
-        
+
     menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(unit_om));
     menu_item = gtk_menu_get_active(GTK_MENU(menu));
     unit = GPOINTER_TO_INT(OBJECT_GET_DATA(menu_item, "size_unit"));
@@ -526,17 +485,18 @@ capture_prep(void)
 
                 *capture_fr, *capture_vb,
                 *if_hb, *if_cb, *if_lb,
+                *if_ip_hb, *if_ip_lb,
                 *linktype_hb, *linktype_lb, *linktype_om,
                 *snap_hb, *snap_cb, *snap_sb, *snap_lb,
                 *promisc_cb,
-                *filter_hb, *filter_bt, *filter_te,
+                *filter_hb, *filter_bt, *filter_te, *filter_cm,
 
                 *file_fr, *file_vb,
                 *file_hb, *file_bt, *file_lb, *file_te,
-                *multi_tb, *multi_files_on_cb, 
+                *multi_tb, *multi_files_on_cb,
                 *ring_filesize_cb, *ring_filesize_sb, *ring_filesize_om,
                 *file_duration_cb, *file_duration_sb, *file_duration_om,
-                *ringbuffer_nbf_cb, *ringbuffer_nbf_sb, *ringbuffer_nbf_lb, 
+                *ringbuffer_nbf_cb, *ringbuffer_nbf_sb, *ringbuffer_nbf_lb,
                 *stop_files_cb, *stop_files_sb, *stop_files_lb,
 
                 *limit_fr, *limit_vb, *limit_tb,
@@ -557,7 +517,7 @@ capture_prep(void)
   GtkTooltips   *tooltips;
   GtkAdjustment *snap_adj, *ringbuffer_nbf_adj,
 		*stop_packets_adj, *stop_filesize_adj, *stop_duration_adj, *stop_files_adj, *ring_filesize_adj, *file_duration_adj;
-  GList         *if_list, *combo_list;
+  GList         *if_list, *combo_list, *filter_list;
   int           err;
   int           row;
   char          err_str[PCAP_ERRBUF_SIZE];
@@ -655,11 +615,17 @@ capture_prep(void)
   }
   free_capture_combo_list(combo_list);
   free_interface_list(if_list);
-  gtk_tooltips_set_tip(tooltips, GTK_COMBO(if_cb)->entry, 
+  gtk_tooltips_set_tip(tooltips, GTK_COMBO(if_cb)->entry,
     "Choose which interface (network card) will be used to capture packets from. "
     "Be sure to select the correct one, as it's a common mistake to select the wrong interface.", NULL);
   gtk_box_pack_start(GTK_BOX(if_hb), if_cb, TRUE, TRUE, 6);
 
+  if_ip_hb = gtk_hbox_new(FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(capture_vb), if_ip_hb, FALSE, FALSE, 0);
+
+  if_ip_lb = gtk_label_new("");
+  gtk_box_pack_start(GTK_BOX(if_ip_hb), if_ip_lb, FALSE, FALSE, 6);
+
   /* Linktype row */
   linktype_hb = gtk_hbox_new(FALSE, 3);
   gtk_box_pack_start(GTK_BOX(capture_vb), linktype_hb, FALSE, FALSE, 0);
@@ -671,6 +637,7 @@ capture_prep(void)
   OBJECT_SET_DATA(linktype_om, E_CAP_LT_OM_LABEL_KEY, linktype_lb);
   /* Default to "use the default" */
   OBJECT_SET_DATA(linktype_om, E_CAP_OM_LT_VALUE_KEY, GINT_TO_POINTER(-1));
+  OBJECT_SET_DATA(linktype_om, E_CAP_IFACE_KEY, if_ip_lb);
   set_link_type_list(linktype_om, GTK_COMBO(if_cb)->entry);
   /*
    * XXX - in some cases, this is "multiple link-layer header types", e.g.
@@ -687,7 +654,7 @@ capture_prep(void)
    *
    * We leave it as "multiple link-layer types" for now.
    */
-  gtk_tooltips_set_tip(tooltips, linktype_om, 
+  gtk_tooltips_set_tip(tooltips, linktype_om,
     "The selected interface supports multiple link-layer types; select the desired one.", NULL);
   gtk_box_pack_start (GTK_BOX(linktype_hb), linktype_om, FALSE, FALSE, 0);
   SIGNAL_CONNECT(GTK_ENTRY(GTK_COMBO(if_cb)->entry), "changed",
@@ -703,7 +670,7 @@ capture_prep(void)
   gtk_spin_button_set_value(GTK_SPIN_BUTTON (buffer_size_sb), (gfloat) capture_opts.buffer_size);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (buffer_size_sb), TRUE);
   WIDGET_SET_SIZE(buffer_size_sb, 80, -1);
-  gtk_tooltips_set_tip(tooltips, buffer_size_sb, 
+  gtk_tooltips_set_tip(tooltips, buffer_size_sb,
     "The memory buffer size used while capturing. If you notice packet drops, you can try to increase this size.", NULL);
   gtk_box_pack_start (GTK_BOX(linktype_hb), buffer_size_sb, FALSE, FALSE, 0);
 
@@ -716,7 +683,7 @@ capture_prep(void)
       "Capture packets in _promiscuous mode", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(promisc_cb),
 		capture_opts.promisc_mode);
-  gtk_tooltips_set_tip(tooltips, promisc_cb, 
+  gtk_tooltips_set_tip(tooltips, promisc_cb,
     "Usually a network card will only capture the traffic sent to its own network address. "
     "If you want to capture all traffic that the network card can \"see\", mark this option. "
     "See the FAQ for some more details of capturing packets from a switched network.", NULL);
@@ -730,7 +697,7 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(snap_cb),
 		capture_opts.has_snaplen);
   SIGNAL_CONNECT(snap_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
-  gtk_tooltips_set_tip(tooltips, snap_cb, 
+  gtk_tooltips_set_tip(tooltips, snap_cb,
     "Limit the maximum number of bytes to be captured from each packet. This size includes the "
     "link-layer header and all subsequent headers. ", NULL);
   gtk_box_pack_start(GTK_BOX(snap_hb), snap_cb, FALSE, FALSE, 0);
@@ -753,21 +720,31 @@ capture_prep(void)
   filter_bt = BUTTON_NEW_FROM_STOCK(ETHEREAL_STOCK_CAPTURE_FILTER_ENTRY);
   SIGNAL_CONNECT(filter_bt, "clicked", capture_filter_construct_cb, NULL);
   SIGNAL_CONNECT(filter_bt, "destroy", filter_button_destroy_cb, NULL);
-  gtk_tooltips_set_tip(tooltips, filter_bt, 
+  gtk_tooltips_set_tip(tooltips, filter_bt,
     "Select a capture filter to reduce the amount of packets to be captured. "
-    "See \"Capture Filters\" in the online help for further information how to use it.", 
+    "See \"Capture Filters\" in the online help for further information how to use it.",
     NULL);
   gtk_box_pack_start(GTK_BOX(filter_hb), filter_bt, FALSE, FALSE, 3);
 
-  filter_te = gtk_entry_new();
-  if (cfile.cfilter) gtk_entry_set_text(GTK_ENTRY(filter_te), cfile.cfilter);
-  OBJECT_SET_DATA(filter_bt, E_FILT_TE_PTR_KEY, filter_te);
-  gtk_tooltips_set_tip(tooltips, filter_te, 
-    "Enter a capture filter to reduce the amount of packets to be captured. "
-    "See \"Capture Filters\" in the online help for further information how to use it.", 
-    NULL);
-  gtk_box_pack_start(GTK_BOX(filter_hb), filter_te, TRUE, TRUE, 3);
+  /* Create the capture filter combo */
+  filter_cm = gtk_combo_new();
 
+  filter_list = OBJECT_GET_DATA(top_level, E_CFILTER_FL_KEY);
+  gtk_combo_disable_activate(GTK_COMBO(filter_cm));
+  gtk_combo_set_case_sensitive(GTK_COMBO(filter_cm), TRUE);
+  OBJECT_SET_DATA(top_level, E_CFILTER_FL_KEY, filter_list);
+  OBJECT_SET_DATA(top_level, E_CFILTER_CM_KEY, filter_cm);
+  filter_te = GTK_COMBO(filter_cm)->entry;
+
+  gtk_combo_set_popdown_strings(GTK_COMBO(filter_cm), filter_list);
+  if (cfile.cfilter)
+    gtk_entry_set_text(GTK_ENTRY(filter_te), cfile.cfilter);
+  gtk_tooltips_set_tip(tooltips, filter_te,
+    "Enter a capture filter to reduce the amount of packets to be captured. "
+    "See \"Capture Filters\" in the online help for further information how to use it.",
+    NULL);
+  WIDGET_SET_SIZE(filter_cm, 400, -1);
+  gtk_box_pack_start(GTK_BOX(filter_hb), filter_cm, FALSE, FALSE, 3);
   main_hb = gtk_hbox_new(FALSE, 5);
   gtk_container_border_width(GTK_CONTAINER(main_hb), 0);
   gtk_container_add(GTK_CONTAINER(main_vb), main_hb);
@@ -780,6 +757,8 @@ capture_prep(void)
   gtk_container_border_width(GTK_CONTAINER(right_vb), 0);
   gtk_box_pack_start(GTK_BOX(main_hb), right_vb, FALSE, FALSE, 0);
 
+  /* let an eventually capture filters dialog know the text entry to fill in */
+  OBJECT_SET_DATA(filter_bt, E_FILT_TE_PTR_KEY, filter_te);
 
   /* Capture file-related options frame */
   file_fr = gtk_frame_new("Capture File(s)");
@@ -797,14 +776,14 @@ capture_prep(void)
   gtk_box_pack_start(GTK_BOX(file_hb), file_lb, FALSE, FALSE, 3);
 
   file_te = gtk_entry_new();
-  gtk_tooltips_set_tip(tooltips, file_te, 
+  gtk_tooltips_set_tip(tooltips, file_te,
     "Enter the file name to which captured data will be written. "
     "If you don't enter something here, a temporary file will be used.",
     NULL);
   gtk_box_pack_start(GTK_BOX(file_hb), file_te, TRUE, TRUE, 3);
 
   file_bt = BUTTON_NEW_FROM_STOCK(ETHEREAL_STOCK_BROWSE);
-  gtk_tooltips_set_tip(tooltips, file_bt, 
+  gtk_tooltips_set_tip(tooltips, file_bt,
     "Select a file to which captured data will be written, "
     "instead of entering the file name directly. ",
     NULL);
@@ -825,7 +804,7 @@ capture_prep(void)
 		capture_opts.multi_files_on);
   SIGNAL_CONNECT(multi_files_on_cb, "toggled", capture_prep_adjust_sensitivity,
                  cap_open_w);
-  gtk_tooltips_set_tip(tooltips, multi_files_on_cb, 
+  gtk_tooltips_set_tip(tooltips, multi_files_on_cb,
     "Instead of using a single capture file, multiple files will be created. "
     "The generated file names will contain an incrementing number and the start time of the capture.", NULL);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), multi_files_on_cb, 0, 1, row, row+1);
@@ -836,7 +815,7 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ring_filesize_cb),
 		capture_opts.has_autostop_filesize);
   SIGNAL_CONNECT(ring_filesize_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
-  gtk_tooltips_set_tip(tooltips, ring_filesize_cb, 
+  gtk_tooltips_set_tip(tooltips, ring_filesize_cb,
     "If the selected file size is exceeded, capturing switches to the next file.",
     NULL);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), ring_filesize_cb, 0, 1, row, row+1);
@@ -860,9 +839,9 @@ capture_prep(void)
   file_duration_cb = gtk_check_button_new_with_label("Next file every");
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(file_duration_cb),
 			      capture_opts.has_file_duration);
-  SIGNAL_CONNECT(file_duration_cb, "toggled", 
+  SIGNAL_CONNECT(file_duration_cb, "toggled",
 		 capture_prep_adjust_sensitivity, cap_open_w);
-  gtk_tooltips_set_tip(tooltips, file_duration_cb, 
+  gtk_tooltips_set_tip(tooltips, file_duration_cb,
     "If the selected duration is exceeded, capturing switches to the next file.",
     NULL);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), file_duration_cb, 0, 1, row, row+1);
@@ -886,7 +865,7 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ringbuffer_nbf_cb),
 		capture_opts.has_ring_num_files);
   SIGNAL_CONNECT(ringbuffer_nbf_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
-  gtk_tooltips_set_tip(tooltips, ringbuffer_nbf_cb, 
+  gtk_tooltips_set_tip(tooltips, ringbuffer_nbf_cb,
     "After capturing has switched to the next file and the given number of files has exceeded, "
     "the oldest file will be removed.",
     NULL);
@@ -910,7 +889,7 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stop_files_cb),
 		capture_opts.has_autostop_files);
   SIGNAL_CONNECT(stop_files_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
-  gtk_tooltips_set_tip(tooltips, stop_files_cb, 
+  gtk_tooltips_set_tip(tooltips, stop_files_cb,
     "Stop capturing after the given number of \"file switches\" have been done.", NULL);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), stop_files_cb, 0, 1, row, row+1);
 
@@ -946,7 +925,7 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stop_packets_cb),
 		capture_opts.has_autostop_packets);
   SIGNAL_CONNECT(stop_packets_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
-  gtk_tooltips_set_tip(tooltips, stop_packets_cb, 
+  gtk_tooltips_set_tip(tooltips, stop_packets_cb,
     "Stop capturing after the given number of packets have been captured.", NULL);
   gtk_table_attach_defaults(GTK_TABLE(limit_tb), stop_packets_cb, 0, 1, row, row+1);
 
@@ -967,7 +946,7 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stop_filesize_cb),
 		capture_opts.has_autostop_filesize);
   SIGNAL_CONNECT(stop_filesize_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
-  gtk_tooltips_set_tip(tooltips, stop_filesize_cb, 
+  gtk_tooltips_set_tip(tooltips, stop_filesize_cb,
     "Stop capturing after the given amount of capture data has been captured.", NULL);
   gtk_table_attach_defaults(GTK_TABLE(limit_tb), stop_filesize_cb, 0, 1, row, row+1);
 
@@ -991,7 +970,7 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stop_duration_cb),
 		capture_opts.has_autostop_duration);
   SIGNAL_CONNECT(stop_duration_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
-  gtk_tooltips_set_tip(tooltips, stop_duration_cb, 
+  gtk_tooltips_set_tip(tooltips, stop_duration_cb,
     "Stop capturing after the given time is exceeded.", NULL);
   gtk_table_attach_defaults(GTK_TABLE(limit_tb), stop_duration_cb, 0, 1, row, row+1);
 
@@ -1023,7 +1002,7 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(sync_cb),
 		capture_opts.sync_mode);
   SIGNAL_CONNECT(sync_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
-  gtk_tooltips_set_tip(tooltips, sync_cb, 
+  gtk_tooltips_set_tip(tooltips, sync_cb,
     "Using this option will show the captured packets immediately on the main screen. "
     "Please note: this will slow down capturing, so increased packet drops might appear.", NULL);
   gtk_container_add(GTK_CONTAINER(display_vb), sync_cb);
@@ -1032,7 +1011,7 @@ capture_prep(void)
   auto_scroll_cb = CHECK_BUTTON_NEW_WITH_MNEMONIC(
 		"_Automatic scrolling in live capture", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(auto_scroll_cb), auto_scroll_live);
-  gtk_tooltips_set_tip(tooltips, auto_scroll_cb, 
+  gtk_tooltips_set_tip(tooltips, auto_scroll_cb,
     "This will scroll the \"Packet List\" automatically to the latest captured packet, "
     "when the \"Update List of packets in real time\" option is used.", NULL);
   gtk_container_add(GTK_CONTAINER(display_vb), auto_scroll_cb);
@@ -1041,7 +1020,7 @@ capture_prep(void)
   hide_info_cb = CHECK_BUTTON_NEW_WITH_MNEMONIC(
 		"_Hide capture info dialog", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(hide_info_cb), !capture_opts.show_info);
-  gtk_tooltips_set_tip(tooltips, hide_info_cb, 
+  gtk_tooltips_set_tip(tooltips, hide_info_cb,
     "Hide the capture info dialog while capturing.", NULL);
   gtk_container_add(GTK_CONTAINER(display_vb), hide_info_cb);
 
@@ -1057,7 +1036,7 @@ capture_prep(void)
 		"Enable _MAC name resolution", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_resolv_cb),
 		g_resolv_flags & RESOLV_MAC);
-  gtk_tooltips_set_tip(tooltips, m_resolv_cb, 
+  gtk_tooltips_set_tip(tooltips, m_resolv_cb,
     "Perform MAC layer name resolution while capturing.", NULL);
   gtk_container_add(GTK_CONTAINER(resolv_vb), m_resolv_cb);
 
@@ -1065,7 +1044,7 @@ capture_prep(void)
 		"Enable _network name resolution", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(n_resolv_cb),
 		g_resolv_flags & RESOLV_NETWORK);
-  gtk_tooltips_set_tip(tooltips, n_resolv_cb, 
+  gtk_tooltips_set_tip(tooltips, n_resolv_cb,
     "Perform network layer name resolution while capturing.", NULL);
   gtk_container_add(GTK_CONTAINER(resolv_vb), n_resolv_cb);
 
@@ -1073,7 +1052,7 @@ capture_prep(void)
 		"Enable _transport name resolution", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(t_resolv_cb),
 		g_resolv_flags & RESOLV_TRANSPORT);
-  gtk_tooltips_set_tip(tooltips, t_resolv_cb, 
+  gtk_tooltips_set_tip(tooltips, t_resolv_cb,
     "Perform transport layer name resolution while capturing.", NULL);
   gtk_container_add(GTK_CONTAINER(resolv_vb), t_resolv_cb);
 
@@ -1083,18 +1062,22 @@ capture_prep(void)
 
   ok_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_OK);
   SIGNAL_CONNECT(ok_bt, "clicked", capture_prep_ok_cb, cap_open_w);
-  gtk_tooltips_set_tip(tooltips, ok_bt, 
+  gtk_tooltips_set_tip(tooltips, ok_bt,
     "Start the capture process.", NULL);
 
   cancel_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
-  gtk_tooltips_set_tip(tooltips, cancel_bt, 
+  gtk_tooltips_set_tip(tooltips, cancel_bt,
     "Cancel and exit dialog.", NULL);
   window_set_cancel_button(cap_open_w, cancel_bt, window_cancel_button_cb);
 
   help_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_HELP);
-  gtk_tooltips_set_tip(tooltips, help_bt, 
+  gtk_tooltips_set_tip(tooltips, help_bt,
     "Show help about capturing.", NULL);
+#ifdef ETHEREAL_EUG_DIR
+  SIGNAL_CONNECT(help_bt, "clicked", url_page_cb, HELP_CAPTURE_OPTIONS_DIALOG);
+#else
   SIGNAL_CONNECT(help_bt, "clicked", help_topic_cb, "Capturing");
+#endif
 
   gtk_widget_grab_default(ok_bt);
 
@@ -1176,19 +1159,22 @@ capture_prep(void)
   SIGNAL_CONNECT(cap_open_w, "delete_event", window_delete_event_cb, NULL);
   SIGNAL_CONNECT(cap_open_w, "destroy", capture_prep_destroy_cb, NULL);
 
+  /* Give the initial focus to the "Filter" entry box. */
+  gtk_widget_grab_focus(filter_te);
+
   gtk_widget_show_all(cap_open_w);
   window_present(cap_open_w);
 }
 
-static void 
+static void
 capture_prep_answered_cb(gpointer dialog _U_, gint btn, gpointer data)
 {
     switch(btn) {
-    case(ESD_BTN_YES):
+    case(ESD_BTN_SAVE):
         /* save file first */
         file_save_as_cmd(after_save_capture_dialog, data);
         break;
-    case(ESD_BTN_NO):
+    case(ESD_BTN_DONT_SAVE):
         capture_prep();
         break;
     case(ESD_BTN_CANCEL):
@@ -1205,9 +1191,9 @@ capture_prep_cb(GtkWidget *w _U_, gpointer d _U_)
 
   if((cfile.state != FILE_CLOSED) && !cfile.user_saved && prefs.gui_ask_unsaved) {
     /* user didn't saved his current file, ask him */
-    dialog = simple_dialog(ESD_TYPE_CONFIRMATION, ESD_BTNS_YES_NO_CANCEL,
+    dialog = simple_dialog(ESD_TYPE_CONFIRMATION, ESD_BTNS_SAVE_DONTSAVE_CANCEL,
                 PRIMARY_TEXT_START "Save capture file before starting a new capture?" PRIMARY_TEXT_END "\n\n"
-                "If you start a new capture without saving, your current capture data will be discarded.");
+                "If you start a new capture without saving, your current capture data will\nbe discarded.");
     simple_dialog_set_cb(dialog, capture_prep_answered_cb, NULL);
   } else {
     /* unchanged file, just capture a new one */
@@ -1229,13 +1215,13 @@ select_link_type_cb(GtkWidget *w, gpointer data)
 static void
 capture_prep_file_cb(GtkWidget *file_bt, GtkWidget *file_te)
 {
-    file_selection_browse(file_bt, file_te, "Ethereal: Specify a Capture File", FILE_SELECTION_OPEN);
+    file_selection_browse(file_bt, file_te, "Ethereal: Specify a Capture File", FILE_SELECTION_WRITE_BROWSE);
 }
 
 
 static void
 capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
-  GtkWidget *if_cb, *snap_cb, *snap_sb, *promisc_cb, *filter_te,
+  GtkWidget *if_cb, *snap_cb, *snap_sb, *promisc_cb, *filter_te, *filter_cm,
             *file_te, *multi_files_on_cb, *ringbuffer_nbf_sb, *ringbuffer_nbf_cb,
             *linktype_om, *sync_cb, *auto_scroll_cb, *hide_info_cb,
             *stop_packets_cb, *stop_packets_sb,
@@ -1266,7 +1252,8 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
   buffer_size_sb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_BUFFER_SIZE_SB_KEY);
 #endif
   promisc_cb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_PROMISC_KEY);
-  filter_te = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_FILT_KEY);
+  filter_cm = OBJECT_GET_DATA(top_level, E_CFILTER_CM_KEY);
+  filter_te = GTK_COMBO(filter_cm)->entry;
   file_te   = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_FILE_TE_KEY);
   multi_files_on_cb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_MULTI_FILES_ON_CB_KEY);
   ringbuffer_nbf_cb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_RING_NBF_CB_KEY);
@@ -1313,7 +1300,7 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
       GPOINTER_TO_INT(OBJECT_GET_DATA(linktype_om, E_CAP_OM_LT_VALUE_KEY));
 
 #ifdef _WIN32
-  capture_opts.buffer_size = 
+  capture_opts.buffer_size =
     gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(buffer_size_sb));
 #endif
 
@@ -1392,7 +1379,7 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
     g_resolv_flags |= RESOLV_TRANSPORT;
 
   capture_opts.has_ring_num_files =
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ringbuffer_nbf_cb));  
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ringbuffer_nbf_cb));
 
   capture_opts.ring_num_files =
     gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ringbuffer_nbf_sb));
@@ -1422,7 +1409,7 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
           PRIMARY_TEXT_START "Multiple files: Requested filesize too large!\n\n" PRIMARY_TEXT_END
           "The setting \"Next file every x byte(s)\" can't be greater than %u bytes (2GB).", G_MAXINT);
         return;
-      }        
+      }
     }
 
     /* test if the settings are ok for a ringbuffer */
@@ -1452,7 +1439,7 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
           PRIMARY_TEXT_START "Stop Capture: Requested filesize too large!\n\n" PRIMARY_TEXT_END
           "The setting \"... after x byte(s)\" can't be greater than %u bytes (2GB).", G_MAXINT);
         return;
-      }        
+      }
     }
   }
 
@@ -1473,7 +1460,11 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
 
   window_destroy(GTK_WIDGET(parent_w));
 
-  do_capture(save_file);
+  if (do_capture(save_file)) {
+    /* The capture succeeded, which means the capture filter syntax is
+       valid; add this capture filter to the recent capture filter list. */
+    cfilter_combo_add_recent(cfile.cfilter);
+  }
   if (save_file != NULL)
     g_free(save_file);
 }
@@ -1616,7 +1607,7 @@ capture_prep_adjust_sensitivity(GtkWidget *tb _U_, gpointer parent_w)
     gtk_widget_set_sensitive(GTK_WIDGET(stop_filesize_om), FALSE);
 
     gtk_widget_set_sensitive(GTK_WIDGET(stop_files_cb), TRUE);
-    gtk_widget_set_sensitive(GTK_WIDGET(stop_files_sb), 
+    gtk_widget_set_sensitive(GTK_WIDGET(stop_files_sb),
           gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stop_files_cb)));
     gtk_widget_set_sensitive(GTK_WIDGET(stop_files_lb),
           gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stop_files_cb)));
