@@ -1,7 +1,7 @@
 /* capture_prefs.c
  * Dialog box for capture preferences
  *
- * $Id: capture_prefs.c,v 1.38 2004/06/20 14:48:23 ulfl Exp $
+ * $Id: capture_prefs.c 12115 2004-09-27 22:55:15Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,7 +35,7 @@
 #include "globals.h"
 #include "capture_prefs.h"
 #include "gtkglobals.h"
-#include "prefs.h"
+#include <epan/prefs.h>
 #include "prefs_dlg.h"
 #include "ui_util.h"
 #include "dlg_utils.h"
@@ -638,7 +638,7 @@ ifopts_options_add(GtkCList *clist, if_info_t *if_info)
 		if (if_info->description != NULL)
 			text[1] = g_strdup(if_info->description);
 		else
-			text[1] = "";
+			text[1] = g_strdup("");
 		/* set empty description */
 		text[2] = g_strdup("");
 		/* check if interface is "hidden" */
@@ -662,7 +662,7 @@ ifopts_options_add(GtkCList *clist, if_info_t *if_info)
 		if (if_info->description != NULL)
 			text[1] = g_strdup(if_info->description);
 		else
-			text[1] = "";
+			text[1] = g_strdup("");
 		/* set empty description */
 		text[2] = g_strdup("");
 		/* interface is not "hidden" */
