@@ -1,7 +1,7 @@
 /* color_dlg.c
  * Definitions for dialog boxes for color filters
  *
- * $Id: color_dlg.c,v 1.52 2004/06/05 10:13:37 ulfl Exp $
+ * $Id: color_dlg.c 12115 2004-09-27 22:55:15Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,7 +36,6 @@
 #include "colors.h"
 #include "color_filters.h"
 #include "color_dlg.h"
-#include "color_utils.h"
 #include "file.h"
 #include <epan/dfilter/dfilter.h>
 #include "simple_dialog.h"
@@ -44,10 +43,10 @@
 #include "ui_util.h"
 #include "dfilter_expr_dlg.h"
 #include "compat_macros.h"
-#include "filter_prefs.h"
+#include "filter_dlg.h"
 #include "file_dlg.h"
 #include "gtkglobals.h"
-#include "prefs.h"
+#include <epan/prefs.h>
 
 static GtkWidget* colorize_dialog_new(char *filter);
 static void add_filter_to_list(gpointer filter_arg, gpointer list_arg);
@@ -74,9 +73,11 @@ static void color_apply_cb(GtkButton *button, gpointer user_data);
 static void color_clear_cb(GtkWidget *button, gpointer user_data);
 static void color_import_cb(GtkButton *button, gpointer user_data );
 
-static void edit_color_filter_dialog_new(GtkWidget *color_filters,
-                                         GtkWidget **colorize_filter_name,
-                                         GtkWidget **colorize_filter_text);
+static void edit_color_filter_dialog(GtkWidget *color_filters,
+                                     GtkWidget **colorize_filter_name,
+                                     GtkWidget **colorize_filter_text,
+                                     guchar is_new_filter);
+
 #if GTK_MAJOR_VERSION < 2
 static void edit_color_filter_destroy_cb(GtkObject *object, gpointer user_data);
 #else
@@ -85,6 +86,7 @@ static void edit_color_filter_destroy_cb(GObject *object, gpointer user_data);
 static void edit_color_filter_fg_cb(GtkButton *button, gpointer user_data);
 static void edit_color_filter_bg_cb(GtkButton *button, gpointer user_data);
 static void edit_color_filter_ok_cb(GtkButton *button, gpointer user_data);
+static void edit_new_color_filter_cancel_cb(GtkButton *button, gpointer user_data);
 
 static GtkWidget* color_sel_win_new(color_filter_t *colorf, gboolean);
 static void color_sel_ok_cb(GtkButton *button, gpointer user_data);
@@ -893,6 +895,7 @@ create_new_color_filter(GtkButton *button, char *filter)
 {
   color_filter_t   *colorf;
   GtkStyle         *style;
+  color_t          bg_color, fg_color;
   GtkWidget        *color_filters;
 #if GTK_MAJOR_VERSION >= 2
   GtkTreeSelection *sel;
@@ -909,17 +912,19 @@ create_new_color_filter(GtkButton *button, char *filter)
 
   /* Use the default background and foreground colors as the colors. */
   style = gtk_widget_get_style(packet_list);
-  colorf = new_color_filter("name", filter, &style->base[GTK_STATE_NORMAL],
-                            &style->text[GTK_STATE_NORMAL]); /* Adds at end! */
+  gdkcolor_to_color_t(&bg_color, &style->base[GTK_STATE_NORMAL]);
+  gdkcolor_to_color_t(&fg_color, &style->text[GTK_STATE_NORMAL]);
+  colorf = new_color_filter("name", filter, &bg_color, &fg_color); /* Adds at end! */
 
   color_add_colorf(color_filters, colorf);
 
-  edit_color_filter_dialog_new(color_filters, &filt_name_entry,
-                               &filt_text_entry);
+  edit_color_filter_dialog(color_filters, &filt_name_entry, &filt_text_entry, TRUE);
   
   /* Show the (in)validity of the default filter string */
   filter_te_syntax_check_cb(filt_text_entry);
 
+
+
 #if GTK_MAJOR_VERSION >= 2
   gtk_widget_grab_focus(color_filters);
 #endif
@@ -941,8 +946,7 @@ color_props_cb(GtkButton *button, gpointer user_data _U_)
 
   color_filters = (GtkWidget *)OBJECT_GET_DATA(button, COLOR_FILTERS_CL);
   g_assert(row_selected != -1);
-  edit_color_filter_dialog_new(color_filters, &filt_name_entry,
-                               &filt_text_entry);
+  edit_color_filter_dialog(color_filters, &filt_name_entry, &filt_text_entry, FALSE);
 }
 
 /* Delete a color from the list. */
@@ -1122,9 +1126,10 @@ color_apply_cb(GtkButton *button _U_, gpointer user_data _U_)
 /* Create an "Edit Color Filter" dialog for a given color filter, and
    associate it with that color filter. */
 static void
-edit_color_filter_dialog_new(GtkWidget *color_filters,
-                             GtkWidget **colorize_filter_name,
-                             GtkWidget **colorize_filter_text)
+edit_color_filter_dialog(GtkWidget *color_filters,
+                         GtkWidget **colorize_filter_name,
+                         GtkWidget **colorize_filter_text,
+                         guchar is_new_filter)
 {
     color_filter_t *colorf;
     GtkWidget      *edit_dialog;
@@ -1266,7 +1271,6 @@ edit_color_filter_dialog_new(GtkWidget *color_filters,
     gtk_tooltips_set_tip (tooltips, edit_color_filter_ok, ("Accept filter color change"), NULL);
 
     edit_color_filter_cancel = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
-    window_set_cancel_button(edit_dialog, edit_color_filter_cancel, window_cancel_button_cb);
     gtk_tooltips_set_tip (tooltips, edit_color_filter_cancel, ("Reject filter color change"), NULL);
 
     gtk_widget_grab_default(edit_color_filter_ok);
@@ -1282,6 +1286,16 @@ edit_color_filter_dialog_new(GtkWidget *color_filters,
     OBJECT_SET_DATA(edit_color_filter_ok, COLOR_FILTER, colorf);
     SIGNAL_CONNECT(edit_color_filter_ok, "clicked", edit_color_filter_ok_cb, edit_dialog);
 
+    /* set callback to delete new filters if cancel chosen */
+    if (is_new_filter)
+    {
+        OBJECT_SET_DATA(edit_color_filter_cancel, COLOR_FILTERS_CL, color_filters);
+        SIGNAL_CONNECT(edit_color_filter_cancel, "clicked",
+                       edit_new_color_filter_cancel_cb, edit_dialog);
+    }
+    /* escape will select cancel */
+    window_set_cancel_button(edit_dialog, edit_color_filter_cancel, window_cancel_button_cb);
+
     SIGNAL_CONNECT(edit_dialog, "delete_event", window_delete_event_cb, NULL);
 
     gtk_widget_show_all(edit_dialog);
@@ -1449,6 +1463,14 @@ edit_color_filter_ok_cb                (GtkButton       *button,
     }
 }
 
+/* reject new color filter addition */
+static void
+edit_new_color_filter_cancel_cb(GtkButton *button, gpointer user_data _U_)
+{
+    /* Delete the entry.  N.B. this already destroys the edit_dialog window. */
+    color_delete(num_of_filters-1, (GtkWidget*)OBJECT_GET_DATA(button, COLOR_FILTERS_CL));
+}
+
 static GtkWidget*
 color_sel_win_new(color_filter_t *colorf, gboolean is_bg)
 {
