@@ -1,6 +1,6 @@
 /* decode_as_dlg.c
  *
- * $Id: decode_as_dlg.c,v 1.41 2004/06/10 09:46:26 guy Exp $
+ * $Id: decode_as_dlg.c 12297 2004-10-14 16:23:11Z tuexen $
  *
  * Routines to modify dissector tables on the fly.
  *
@@ -34,10 +34,11 @@
 #include "globals.h"
 #include "simple_dialog.h"
 #include <epan/packet.h>
-#include "ipproto.h"
+#include <epan/ipproto.h>
 #include "ui_util.h"
 #include <epan/epan_dissect.h>
 #include "compat_macros.h"
+#include "decode_as_dcerpc.h"
 
 #undef DEBUG
 
@@ -45,18 +46,6 @@
 /*                Typedefs & Enums                */
 /**************************************************/
 
-/*
- * Enum used to track which radio button is currently selected in the
- * dialog. These buttons are labeled "Decode" and "Do not decode".
- */
-enum action_type {
-    /* The "Decode" button is currently selected. */
-    E_DECODE_YES,
-
-    /* The "Do not decode" button is currently selected. */
-    E_DECODE_NO
-};
-
 /*
  * Enum used to track which transport layer port menu item is
  * currently selected in the dialog.  These items are labeled "source",
@@ -68,7 +57,9 @@ enum srcdst_type {
     /* The "destination port" menu item is currently selected. */
     E_DECODE_DPORT,
     /* The "source/destination port" menu item is currently selected. */
-    E_DECODE_BPORT
+    E_DECODE_BPORT,
+    /* For SCTP only. This MUST be the last entry! */
+    E_DECODE_PPID
 };
 
 #define E_DECODE_MIN_HEIGHT 300
@@ -76,22 +67,9 @@ enum srcdst_type {
 
 #define E_MENU_SRCDST "menu_src_dst"
 
-#define E_PAGE_ACTION "notebook_page_action"
-#define E_PAGE_LIST   "notebook_page_list"
-#define E_PAGE_TABLE  "notebook_page_table_name"
-#define E_PAGE_TITLE  "notebook_page_title"
-#define E_PAGE_VALUE  "notebook_page_value"
-
-/*
- * Columns for a "Select" list.
- * Note that most of these columns aren't displayed; they're attached
- * to the row of the table as additional information.
- */
-#define E_LIST_S_PROTO_NAME 0
-#define E_LIST_S_TABLE	    1
-/* The following is for debugging in decode_add_to_list */
-#define E_LIST_S_MAX	    E_LIST_S_TABLE
-#define E_LIST_S_COLUMNS   (E_LIST_S_MAX + 1)
+#define E_PAGE_DPORT "dport"
+#define E_PAGE_SPORT "sport"
+#define E_PAGE_PPID  "ppid"
 
 /*
  * Columns for a "Display" list
@@ -129,7 +107,7 @@ static GtkWidget *decode_show_w = NULL;
  * selected the "Decode" radio button.  When the "Do not decode"
  * button is selected these items should be dimmed.
  */
-static GSList *decode_dimmable = NULL;
+GSList *decode_dimmable = NULL;
 
 /*
  * Remember the "action" radio button that is currently selected in
@@ -137,10 +115,21 @@ static GSList *decode_dimmable = NULL;
  * modified in a callback routine, and read in the routine that
  * handles a click in the "OK" button for the dialog.
  */
-static enum action_type	requested_action = -1;
+enum action_type	requested_action = -1;
+
 
 /**************************************************/
-/*            Resett Changed Dissectors           */
+/*            Global Functions                    */
+/**************************************************/
+
+/* init this module */
+void decode_as_init(void) {
+
+    decode_dcerpc_init();
+}
+
+/**************************************************/
+/*            Reset Changed Dissectors            */
 /**************************************************/
 
 /*
@@ -227,6 +216,85 @@ decode_build_reset_list (gchar *table_name, ftenum_t selector_type,
 /*             Show Changed Dissectors            */
 /**************************************************/
 
+#if GTK_MAJOR_VERSION >= 2
+#define SORT_ALPHABETICAL 0
+
+static gint
+sort_iter_compare_func (GtkTreeModel *model,
+GtkTreeIter *a,
+GtkTreeIter *b,
+gpointer userdata)
+{
+    gint sortcol = GPOINTER_TO_INT(userdata);
+    gint ret = 0;
+    switch (sortcol)
+    {
+        case SORT_ALPHABETICAL:
+        {
+        gchar *name1, *name2;
+        gtk_tree_model_get(model, a, 0, &name1, -1);
+        gtk_tree_model_get(model, b, 0, &name2, -1);
+        if (name1 == NULL || name2 == NULL)
+        {
+            if (name1 == NULL && name2 == NULL)
+                break; /* both equal => ret = 0 */
+            ret = (name1 == NULL) ? -1 : 1;
+        }
+        else
+        {
+            ret = g_ascii_strcasecmp(name1,name2);
+        }
+        g_free(name1);
+        g_free(name2);
+        }
+        break;
+        default:
+        g_return_val_if_reached(0);
+    }
+    return ret;
+}
+#endif
+
+
+void
+decode_add_to_show_list (
+gpointer list_data, 
+gchar *table_name, 
+gchar *selector_name, 
+gchar *initial_proto_name, 
+gchar *current_proto_name)
+{
+    gchar     *text[E_LIST_D_COLUMNS];
+#if GTK_MAJOR_VERSION < 2
+    GtkCList  *clist;
+    gint       row;
+#else
+    GtkListStore *store;
+    GtkTreeIter   iter;
+#endif
+
+#if GTK_MAJOR_VERSION < 2
+    clist = (GtkCList *)list_data;
+#else
+    store = (GtkListStore *)list_data;
+#endif
+
+    text[E_LIST_D_TABLE] = table_name;
+    text[E_LIST_D_SELECTOR] = selector_name;
+    text[E_LIST_D_INITIAL] = initial_proto_name;
+    text[E_LIST_D_CURRENT] = current_proto_name;
+#if GTK_MAJOR_VERSION < 2
+    row = gtk_clist_prepend(clist, text);
+#else
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter, E_LIST_D_TABLE, text[E_LIST_D_TABLE],
+                       E_LIST_D_SELECTOR, text[E_LIST_D_SELECTOR],
+                       E_LIST_D_INITIAL, text[E_LIST_D_INITIAL],
+                       E_LIST_D_CURRENT, text[E_LIST_D_CURRENT], -1);
+#endif
+}
+
+
 /*
  * This routine creates one entry in the list of protocol dissector
  * that have been changed.  It is called by the g_hash_foreach routine
@@ -250,25 +318,13 @@ static void
 decode_build_show_list (gchar *table_name, ftenum_t selector_type,
 			gpointer key, gpointer value, gpointer user_data)
 {
-#if GTK_MAJOR_VERSION < 2
-    GtkCList  *clist;
-    gint       row;
-#else
-    GtkListStore *store;
-    GtkTreeIter   iter;
-#endif
     dissector_handle_t current, initial;
-    gchar     *current_proto_name, *initial_proto_name, *text[E_LIST_D_COLUMNS];
+    gchar     *current_proto_name, *initial_proto_name, *selector_name;
     gchar      string1[20];
 
     g_assert(user_data);
     g_assert(value);
 
-#if GTK_MAJOR_VERSION < 2
-    clist = (GtkCList *)user_data;
-#else
-    store = (GtkListStore *)user_data;
-#endif
     current = dtbl_entry_get_handle(value);
     if (current == NULL)
 	current_proto_name = "(none)";
@@ -280,7 +336,6 @@ decode_build_show_list (gchar *table_name, ftenum_t selector_type,
     else
 	initial_proto_name = dissector_handle_get_short_name(initial);
 
-    text[E_LIST_D_TABLE] = get_dissector_table_ui_name(table_name);
     switch (selector_type) {
 
     case FT_UINT8:
@@ -322,29 +377,66 @@ decode_build_show_list (gchar *table_name, ftenum_t selector_type,
 	    g_snprintf(string1, sizeof(string1), "%#o", GPOINTER_TO_UINT(key));
 	    break;
 	}
-	text[E_LIST_D_SELECTOR] = string1;
+	selector_name = string1;
 	break;
 
     case FT_STRING:
     case FT_STRINGZ:
-	text[E_LIST_D_SELECTOR] = key;
+	selector_name = key;
 	break;
 
     default:
 	g_assert_not_reached();
+	selector_name = NULL;
 	break;
     }
-    text[E_LIST_D_INITIAL] = initial_proto_name;
-    text[E_LIST_D_CURRENT] = current_proto_name;
-#if GTK_MAJOR_VERSION < 2
-    row = gtk_clist_prepend(clist, text);
-#else
-    gtk_list_store_append(store, &iter);
-    gtk_list_store_set(store, &iter, E_LIST_D_TABLE, text[E_LIST_D_TABLE],
-                       E_LIST_D_SELECTOR, text[E_LIST_D_SELECTOR],
-                       E_LIST_D_INITIAL, text[E_LIST_D_INITIAL],
-                       E_LIST_D_CURRENT, text[E_LIST_D_CURRENT], -1);
-#endif
+
+    decode_add_to_show_list (
+        user_data, 
+        get_dissector_table_ui_name(table_name),
+        selector_name, 
+        initial_proto_name, 
+        current_proto_name);
+}
+
+
+/* clear all settings */
+static void
+decode_clear_all(void)
+{
+    dissector_delete_item_t *item;
+    GSList *tmp;
+
+    dissector_all_tables_foreach_changed(decode_build_reset_list, NULL);
+
+    for (tmp = dissector_reset_list; tmp; tmp = g_slist_next(tmp)) {
+	item = tmp->data;
+	switch (item->ddi_selector_type) {
+
+	case FT_UINT8:
+	case FT_UINT16:
+	case FT_UINT24:
+	case FT_UINT32:
+	    dissector_reset(item->ddi_table_name, item->ddi_selector.sel_uint);
+	    break;
+
+	case FT_STRING:
+	case FT_STRINGZ:
+	    dissector_reset_string(item->ddi_table_name,
+				   item->ddi_selector.sel_string);
+	    break;
+
+	default:
+	    g_assert_not_reached();
+	}
+	g_free(item);
+    }
+    g_slist_free(dissector_reset_list);
+    dissector_reset_list = NULL;
+
+    decode_dcerpc_reset_all();
+
+    redissect_packets(&cfile);
 }
 
 
@@ -377,47 +469,16 @@ decode_show_ok_cb (GtkWidget *ok_bt _U_, gpointer parent_w)
 static void
 decode_show_clear_cb (GtkWidget *clear_bt _U_, gpointer parent_w)
 {
-    dissector_delete_item_t *item;
-    GSList *tmp;
-
-    dissector_all_tables_foreach_changed(decode_build_reset_list, NULL);
-
-    for (tmp = dissector_reset_list; tmp; tmp = g_slist_next(tmp)) {
-	item = tmp->data;
-	switch (item->ddi_selector_type) {
-
-	case FT_UINT8:
-	case FT_UINT16:
-	case FT_UINT24:
-	case FT_UINT32:
-	    dissector_reset(item->ddi_table_name, item->ddi_selector.sel_uint);
-	    break;
-
-	case FT_STRING:
-	case FT_STRINGZ:
-	    dissector_reset_string(item->ddi_table_name,
-				   item->ddi_selector.sel_string);
-	    break;
-
-	default:
-	    g_assert_not_reached();
-	}
-	g_free(item);
-    }
-    g_slist_free(dissector_reset_list);
-    dissector_reset_list = NULL;
-
-    redissect_packets(&cfile);
+    decode_clear_all();
 
     window_destroy(GTK_WIDGET(parent_w));
 }
 
 
 /*
- * This routine is called when the user clicks the "Close" button in
+ * This routine is called when the user clicks the X at the top right end in
  * the "Decode As:Show..." dialog window.  This routine simply calls the
- * cancel routine as if the user had clicked the cancel button instead
- * of the close button.
+ * ok routine as if the user had clicked the ok button.
  *
  * @param GtkWidget * A pointer to the dialog box.
  *
@@ -461,7 +522,7 @@ decode_show_cb (GtkWidget * w _U_, gpointer data _U_)
 {
     GtkWidget         *main_vb, *bbox, *ok_bt, *clear_bt, *scrolled_window;
     gchar             *titles[E_LIST_D_COLUMNS] = {
-        "Table", "Port", "Initial", "Current"
+        "Table", "Value", "Initial", "Current"
     };
     gint               column;
 #if GTK_MAJOR_VERSION < 2
@@ -520,9 +581,11 @@ decode_show_cb (GtkWidget * w _U_, gpointer data _U_)
 #if GTK_MAJOR_VERSION < 2
 	dissector_all_tables_foreach_changed(decode_build_show_list, list);
 	gtk_clist_sort(list);
+    decode_dcerpc_add_show_list(list);
 #else
 	dissector_all_tables_foreach_changed(decode_build_show_list, store);
 	g_object_unref(G_OBJECT(store));
+    decode_dcerpc_add_show_list(store);
 #endif
 
 	/* Put clist into a scrolled window */
@@ -612,7 +675,7 @@ decode_change_one_dissector(gchar *table_name, guint selector, GtkWidget *list)
 #if GTK_MAJOR_VERSION < 2
     if (!GTK_CLIST(list)->selection)
     {
-	abbrev = "(NULL)";
+	abbrev = NULL;
 	handle = NULL;
     } else {
 	row = GPOINTER_TO_INT(GTK_CLIST(list)->selection->data);
@@ -623,7 +686,7 @@ decode_change_one_dissector(gchar *table_name, guint selector, GtkWidget *list)
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
     if (gtk_tree_selection_get_selected(selection, &model, &iter) == FALSE)
     {
-	abbrev = "(NULL)";
+	abbrev = NULL;
 	handle = NULL;
     } else {
         gtk_tree_model_get(model, &iter, E_LIST_S_PROTO_NAME, &abbrev,
@@ -631,18 +694,18 @@ decode_change_one_dissector(gchar *table_name, guint selector, GtkWidget *list)
     }
 #endif
 
-    if (strcmp(abbrev, "(default)") == 0) {
+    if (abbrev != NULL && strcmp(abbrev, "(default)") == 0) {
 	dissector_reset(table_name, selector);
     } else {
 	dissector_change(table_name, selector, handle);
     }
 #if GTK_MAJOR_VERSION >= 2
-    g_free(abbrev);
+    if (abbrev != NULL)
+	g_free(abbrev);
 #endif
 }
 
 
-
 /**************************************************/
 /* Action routines for the "Decode As..." dialog  */
 /*   - called when the OK button pressed          */
@@ -737,7 +800,8 @@ decode_transport(GtkWidget *notebook_pg)
     GtkWidget *menu, *menuitem;
     GtkWidget *list;
     gchar *table_name;
-    gint requested_srcdst;
+    gint requested_srcdst, requested_port, ppid;
+    gpointer portp;
 
     list = OBJECT_GET_DATA(notebook_pg, E_PAGE_LIST);
     if (requested_action == E_DECODE_NO)
@@ -758,12 +822,34 @@ decode_transport(GtkWidget *notebook_pg)
 #endif
 
     table_name = OBJECT_GET_DATA(notebook_pg, E_PAGE_TABLE);
-    if (requested_srcdst != E_DECODE_DPORT)
-	decode_change_one_dissector(table_name, cfile.edt->pi.srcport, list);
-    if (requested_srcdst != E_DECODE_SPORT)
-	decode_change_one_dissector(table_name, cfile.edt->pi.destport, list);
+    if (requested_srcdst >= E_DECODE_PPID) {
+    	if (requested_srcdst == E_DECODE_PPID)
+    	   ppid = 0;
+        else
+           if (requested_srcdst - E_DECODE_PPID - 1 < MAX_NUMBER_OF_PPIDS)
+             ppid = cfile.edt->pi.ppid[requested_srcdst - E_DECODE_PPID - 1];
+           else 
+             return;
+        decode_change_one_dissector(table_name, ppid, list);
+        return;
+    }
+    if (requested_srcdst != E_DECODE_DPORT) {
+        portp = OBJECT_GET_DATA(notebook_pg, E_PAGE_SPORT);
+        if (portp != NULL) {
+            requested_port = GPOINTER_TO_INT(portp);
+            decode_change_one_dissector(table_name, requested_port, list);
+        }
+    }
+    if (requested_srcdst != E_DECODE_SPORT) {
+        portp = OBJECT_GET_DATA(notebook_pg, E_PAGE_DPORT);
+        if (portp != NULL) {
+            requested_port = GPOINTER_TO_INT(portp);
+            decode_change_one_dissector(table_name, requested_port, list);
+        }
+    }
 }
 
+
 /**************************************************/
 /*      Signals from the "Decode As..." dialog    */
 /**************************************************/
@@ -787,6 +873,7 @@ decode_ok_cb (GtkWidget *ok_bt _U_, gpointer parent_w)
     GtkWidget *notebook, *notebook_pg;
     void (* func)(GtkWidget *);
     gint page_num;
+    void *binding = NULL;
 
     /* Call the right routine for the page that was currently in front. */
     notebook =  OBJECT_GET_DATA(parent_w, E_NOTEBOOK);
@@ -797,6 +884,13 @@ decode_ok_cb (GtkWidget *ok_bt _U_, gpointer parent_w)
     func(notebook_pg);
 
     /* Now destroy the "Decode As" dialog. */
+    notebook_pg = OBJECT_GET_DATA(parent_w, E_PAGE_DCERPC);
+    if(notebook_pg) {
+        binding = OBJECT_GET_DATA(notebook_pg, E_PAGE_BINDING);
+    }
+    if(binding) {
+        decode_dcerpc_binding_free(binding);    
+    }
     window_destroy(GTK_WIDGET(parent_w));
     g_slist_free(decode_dimmable);
     decode_dimmable = NULL;
@@ -834,17 +928,28 @@ decode_apply_cb (GtkWidget *apply_bt _U_, gpointer parent_w)
 }
 
 /*
- * This routine is called when the user clicks the "Cancel" button in
+ * This routine is called when the user clicks the "Close" button in
  * the "Decode As..." dialog window.  This routine then destroys the
  * dialog box and performs other housekeeping functions.
  *
- * @param cancel_bt A pointer to the "Cancel" button.
+ * @param close_bt A pointer to the "Close" button.
  *
  * @param parent_w A pointer to the dialog window.
  */
 static void
-decode_cancel_cb (GtkWidget *cancel_bt _U_, gpointer parent_w)
+decode_close_cb (GtkWidget *close_bt _U_, gpointer parent_w)
 {
+    GtkWidget *notebook_pg = NULL;
+    void *binding = NULL;
+
+
+    notebook_pg = OBJECT_GET_DATA(parent_w, E_PAGE_DCERPC);
+    if(notebook_pg) {
+        binding = OBJECT_GET_DATA(notebook_pg, E_PAGE_BINDING);
+    }
+    if(binding) {
+        decode_dcerpc_binding_free(binding);
+    }
     window_destroy(GTK_WIDGET(parent_w));
     g_slist_free(decode_dimmable);
     decode_dimmable = NULL;
@@ -854,7 +959,7 @@ decode_cancel_cb (GtkWidget *cancel_bt _U_, gpointer parent_w)
 /*
  * This routine is called when the user clicks the "Close" button in
  * the "Decode As..." dialog window.  This routine simply calls the
- * cancel routine as if the user had clicked the cancel button instead
+ * close routine as if the user had clicked the close button instead
  * of the close button.
  *
  * @param decode_w A pointer to the dialog box.
@@ -864,7 +969,7 @@ decode_cancel_cb (GtkWidget *cancel_bt _U_, gpointer parent_w)
 static gboolean
 decode_delete_cb (GtkWidget *decode_w, gpointer dummy _U_)
 {
-    decode_cancel_cb(NULL, decode_w);
+    decode_close_cb(NULL, decode_w);
     return FALSE;
 }
 
@@ -889,6 +994,22 @@ decode_destroy_cb (GtkWidget *win _U_, gpointer user_data _U_)
 }
 
 
+/*
+ * This routine is called when the user clicks the "Clear" button in
+ * the "Decode As..." dialog window.  This routine resets all the
+ * dissector values and performs other housekeeping functions.
+ *
+ * @param GtkWidget * A pointer to the "Clear" button.
+ * @param gpointer A pointer to the dialog window.
+ */
+static void
+decode_clear_cb(GtkWidget *clear_bt _U_, gpointer parent_w _U_)
+{
+    decode_clear_all();
+}
+
+
+
 /**************************************************/
 /*          Dialog setup - radio buttons          */
 /**************************************************/
@@ -1011,12 +1132,46 @@ decode_add_srcdst_menu (GtkWidget *page)
 
     OBJECT_SET_DATA(page, E_MENU_SRCDST, menu);
     gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), menu);
+    OBJECT_SET_DATA(page, E_PAGE_SPORT, GINT_TO_POINTER(cfile.edt->pi.srcport));
+    OBJECT_SET_DATA(page, E_PAGE_DPORT, GINT_TO_POINTER(cfile.edt->pi.destport));
 
     alignment = decode_add_pack_menu(optmenu);
 
     return(alignment);
 }
 
+static GtkWidget *
+decode_add_ppid_menu (GtkWidget *page)
+{
+    GtkWidget *optmenu, *menu, *menuitem;
+    gchar      tmp[100];
+    guint      number_of_ppid;
+    
+    optmenu = gtk_option_menu_new();
+    menu = gtk_menu_new();
+    
+    g_snprintf(tmp, 100, "PPID (%u)", 0);
+    menuitem = gtk_menu_item_new_with_label(tmp);
+    OBJECT_SET_DATA(menuitem, "user_data", GINT_TO_POINTER(E_DECODE_PPID));
+    gtk_menu_append(GTK_MENU(menu), menuitem);
+    gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
+    
+    for(number_of_ppid = 0; number_of_ppid < MAX_NUMBER_OF_PPIDS; number_of_ppid++)
+      if (cfile.edt->pi.ppid[number_of_ppid] != 0) {
+        g_snprintf(tmp, 100, "PPID (%u)", cfile.edt->pi.ppid[number_of_ppid]);
+        menuitem = gtk_menu_item_new_with_label(tmp);
+        OBJECT_SET_DATA(menuitem, "user_data", GINT_TO_POINTER(E_DECODE_PPID + 1 + number_of_ppid));
+        gtk_menu_append(GTK_MENU(menu), menuitem);
+        gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
+      } else
+        break;
+
+    OBJECT_SET_DATA(page, E_MENU_SRCDST, menu);
+    gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), menu);
+
+    return(optmenu);
+}
+
 /*************************************************/
 /*        Dialog setup - list based menus        */
 /*************************************************/
@@ -1063,12 +1218,10 @@ lookup_handle(GtkTreeModel *model, GtkTreePath *path _U_, GtkTreeIter *iter,
  * routine, specifying information about the dissector table and where
  * to store any information generated by this routine.
  */
-static void
-decode_add_to_list (gchar *table_name, gpointer value, gpointer user_data)
+void
+decode_add_to_list (gchar *table_name, gchar *proto_name, gpointer value, gpointer user_data)
 {
-    gchar     *proto_name;
     gchar     *text[E_LIST_S_COLUMNS];
-    dissector_handle_t handle;
 #if GTK_MAJOR_VERSION < 2
     GtkCList  *list;
     gint       row;
@@ -1078,26 +1231,19 @@ decode_add_to_list (gchar *table_name, gpointer value, gpointer user_data)
     GtkTreeIter   iter;
     struct handle_lookup_info hli;
 #endif
-    gint       i;
 
     g_assert(user_data);
     g_assert(value);
 
     list = user_data;
-    handle = value;
-    proto_name = dissector_handle_get_short_name(handle);
 
-    i = dissector_handle_get_protocol_index(handle);
-    if (i >= 0 && !proto_is_protocol_enabled(find_protocol_by_id(i)))
-        return;
-  
 #if GTK_MAJOR_VERSION < 2
-    row = gtk_clist_find_row_from_data(list, handle);
+    row = gtk_clist_find_row_from_data(list, value);
     /* We already have an entry for this handle.
      * XXX - will this ever happen? */
     if (row != -1) return;
 #else
-    hli.handle = handle;
+    hli.handle = value;
     hli.found = FALSE;
     store = GTK_LIST_STORE(gtk_tree_view_get_model(list));
     gtk_tree_model_foreach(GTK_TREE_MODEL(store), lookup_handle, &hli);
@@ -1110,16 +1256,34 @@ decode_add_to_list (gchar *table_name, gpointer value, gpointer user_data)
     text[E_LIST_S_TABLE] = table_name;
 #if GTK_MAJOR_VERSION < 2
     row = gtk_clist_prepend(list, text);
-    gtk_clist_set_row_data(list, row, handle);
+    gtk_clist_set_row_data(list, row, value);
 #else
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter,
                        E_LIST_S_PROTO_NAME, text[E_LIST_S_PROTO_NAME],
                        E_LIST_S_TABLE, text[E_LIST_S_TABLE],
-                       E_LIST_S_TABLE+1, handle, -1);
+                       E_LIST_S_TABLE+1, value, -1);
 #endif
 }
 
+static void
+decode_proto_add_to_list (gchar *table_name, gpointer value, gpointer user_data)
+{
+    gchar     *proto_name;
+    gint       i;
+    dissector_handle_t handle;
+
+
+    handle = value;
+    proto_name = dissector_handle_get_short_name(handle);
+
+    i = dissector_handle_get_protocol_index(handle);
+    if (i >= 0 && !proto_is_protocol_enabled(find_protocol_by_id(i)))
+        return;
+  
+    decode_add_to_list (table_name, proto_name, value, user_data);
+}
+
 
 /*
  * This routine starts the creation of a List on a notebook page.  It
@@ -1134,7 +1298,7 @@ decode_add_to_list (gchar *table_name, gpointer value, gpointer user_data)
  * @param scrolled_win_p Will be filled in with the address of a newly
  * created GtkScrolledWindow.
  */
-static void
+void
 decode_list_menu_start(GtkWidget *page, GtkWidget **list_p,
                        GtkWidget **scrolled_win_p)
 {
@@ -1147,10 +1311,13 @@ decode_list_menu_start(GtkWidget *page, GtkWidget **list_p,
     GtkListStore      *store;
     GtkCellRenderer   *renderer;
     GtkTreeViewColumn *tc;
+    GtkTreeSortable   *sortable;
 #endif
 
 #if GTK_MAJOR_VERSION < 2
     list = GTK_CLIST(gtk_clist_new_with_titles(E_LIST_S_COLUMNS, titles));
+
+    OBJECT_SET_DATA(decode_w, "sctp_list", list);
     gtk_clist_column_titles_passive(list);
 #ifndef DEBUG
     gtk_clist_column_titles_hide(list);
@@ -1163,7 +1330,11 @@ decode_list_menu_start(GtkWidget *page, GtkWidget **list_p,
 #else
     store = gtk_list_store_new(E_LIST_S_COLUMNS+1, G_TYPE_STRING,
                                G_TYPE_STRING, G_TYPE_POINTER);
+    OBJECT_SET_DATA(G_OBJECT(decode_w), "sctp_data", store);
     list = GTK_TREE_VIEW(tree_view_new(GTK_TREE_MODEL(store)));
+    sortable = GTK_TREE_SORTABLE(store);
+    gtk_tree_sortable_set_sort_func(sortable, SORT_ALPHABETICAL, sort_iter_compare_func, GINT_TO_POINTER(SORT_ALPHABETICAL), NULL);
+    gtk_tree_sortable_set_sort_column_id(sortable, SORT_ALPHABETICAL, GTK_SORT_ASCENDING);
     gtk_tree_view_set_headers_clickable(list, FALSE);
 #ifndef DEBUG
     gtk_tree_view_set_headers_visible(list, FALSE);
@@ -1202,7 +1373,7 @@ decode_list_menu_start(GtkWidget *page, GtkWidget **list_p,
  *
  * @param list A pointer the the List to finish.
  */
-static void
+void
 decode_list_menu_finish(GtkWidget *list)
 {
     gchar *text[E_LIST_S_COLUMNS];
@@ -1256,11 +1427,12 @@ decode_add_simple_menu (GtkWidget *page, gchar *table_name)
     GtkWidget *list;
 
     decode_list_menu_start(page, &list, &scrolled_window);
-    dissector_table_foreach_handle(table_name, decode_add_to_list, list);
+    dissector_table_foreach_handle(table_name, decode_proto_add_to_list, list);
     decode_list_menu_finish(list);
     return(scrolled_window);
 }
 
+
 /**************************************************/
 /*                  Dialog setup                  */
 /**************************************************/
@@ -1365,6 +1537,219 @@ decode_add_tcpudp_page (gchar *prompt, gchar *table_name)
     return(page);
 }
 
+void
+decode_sctp_list_menu_start(GtkWidget **list_p, GtkWidget **scrolled_win_p)
+{
+#if GTK_MAJOR_VERSION < 2
+    gchar             *titles[E_LIST_S_COLUMNS] = {"Short Name", "Table Name"};
+    GtkCList          *list;
+    gint               column;
+#else
+    GtkTreeView       *list;
+    GtkListStore      *sctp_store;
+    GtkCellRenderer   *renderer;
+    GtkTreeViewColumn *tc;
+    GtkTreeSortable   *sortable;
+#endif
+
+#if GTK_MAJOR_VERSION < 2
+    list=OBJECT_GET_DATA(decode_w, "sctp_list");
+    gtk_clist_column_titles_passive(list);
+#ifndef DEBUG
+    gtk_clist_column_titles_hide(list);
+    for (column = 1; column < E_LIST_S_COLUMNS; column++)
+        gtk_clist_set_column_visibility (list, column, FALSE);
+#endif
+    for (column = 0; column < E_LIST_S_COLUMNS; column++)
+        gtk_clist_set_column_auto_resize(list, column, TRUE);
+#else
+    sctp_store = OBJECT_GET_DATA(decode_w, "sctp_data");
+    list = GTK_TREE_VIEW(tree_view_new(GTK_TREE_MODEL(sctp_store)));
+    sortable = GTK_TREE_SORTABLE(sctp_store);
+    gtk_tree_sortable_set_sort_func(sortable, SORT_ALPHABETICAL, sort_iter_compare_func, GINT_TO_POINTER(SORT_ALPHABETICAL), NULL);
+    gtk_tree_sortable_set_sort_column_id(sortable, SORT_ALPHABETICAL, GTK_SORT_ASCENDING);
+    gtk_tree_view_set_headers_clickable(list, FALSE);
+#ifndef DEBUG
+    gtk_tree_view_set_headers_visible(list, FALSE);
+#endif
+    renderer = gtk_cell_renderer_text_new();
+    tc = gtk_tree_view_column_new_with_attributes("Short Name", renderer,
+                                                  "text", E_LIST_S_PROTO_NAME,
+                                                  NULL);
+    gtk_tree_view_column_set_sizing(tc, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+    gtk_tree_view_append_column(list, tc);
+#endif
+
+    *scrolled_win_p = scrolled_window_new(NULL, NULL);
+    /* this will result to set the width of the dialog to the required size */
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(*scrolled_win_p), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+#if GTK_MAJOR_VERSION >= 2
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(*scrolled_win_p), GTK_SHADOW_IN);
+    gtk_container_add(GTK_CONTAINER(*scrolled_win_p), GTK_WIDGET(list));
+#endif
+    *list_p = GTK_WIDGET(list);
+}
+
+static void
+decode_sctp_update_ppid_menu(GtkWidget *w _U_, GtkWidget *page)
+{
+    GtkWidget *menu, *menuitem, *list, *scrolled_window, *sctpmenu;
+    gchar      tmp[100];
+    guint      number_of_ppid;
+#if GTK_MAJOR_VERSION < 2
+    GtkCList *sctp_list;
+#else
+    GtkListStore *sctp_store;
+#endif
+
+    menu = gtk_menu_new();
+
+    g_snprintf(tmp, 100, "PPID (%u)", 0);
+    menuitem = gtk_menu_item_new_with_label(tmp);
+    OBJECT_SET_DATA(menuitem, "user_data", GINT_TO_POINTER(E_DECODE_PPID));
+    gtk_menu_append(GTK_MENU(menu), menuitem);
+    gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
+    for(number_of_ppid = 0; number_of_ppid < MAX_NUMBER_OF_PPIDS; number_of_ppid++)
+      if (cfile.edt->pi.ppid[number_of_ppid] != 0) {
+        g_snprintf(tmp, 100, "PPID (%u)", cfile.edt->pi.ppid[number_of_ppid]);
+        menuitem = gtk_menu_item_new_with_label(tmp);
+        OBJECT_SET_DATA(menuitem, "user_data", GINT_TO_POINTER(E_DECODE_PPID + 1 + number_of_ppid));
+        gtk_menu_append(GTK_MENU(menu), menuitem);
+        gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
+      } else
+        break;
+
+    OBJECT_SET_DATA(page, E_MENU_SRCDST, menu);
+    OBJECT_SET_DATA(page, E_PAGE_TABLE, "sctp.ppi");
+    sctpmenu = OBJECT_GET_DATA(decode_w, "user_data");
+    gtk_option_menu_set_menu(GTK_OPTION_MENU(sctpmenu), menu);
+
+#if GTK_MAJOR_VERSION < 2
+    sctp_list = OBJECT_GET_DATA(decode_w, "sctp_list");
+    gtk_clist_clear(sctp_list);
+#else
+    sctp_store = OBJECT_GET_DATA(G_OBJECT(decode_w), "sctp_data");
+    gtk_list_store_clear(sctp_store);
+#endif
+    decode_sctp_list_menu_start(&list, &scrolled_window);
+    dissector_table_foreach_handle("sctp.ppi", decode_proto_add_to_list, list);
+    decode_list_menu_finish(list);
+}
+
+
+static void
+decode_sctp_update_srcdst_menu(GtkWidget *w _U_, GtkWidget *page)
+{
+    GtkWidget  *menu, *menuitem, *scrolled_window, *list, *sctpmenu;
+    gchar      tmp[100];
+#if GTK_MAJOR_VERSION < 2
+    GtkCList	*sctp_list;
+#else
+    GtkListStore *sctp_store;
+#endif
+
+    menu = gtk_menu_new();
+    g_snprintf(tmp, 100, "source (%u)", cfile.edt->pi.srcport);
+    menuitem = gtk_menu_item_new_with_label(tmp);
+    OBJECT_SET_DATA(menuitem, "user_data", GINT_TO_POINTER(E_DECODE_SPORT));
+    gtk_menu_append(GTK_MENU(menu), menuitem);
+    gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
+
+    g_snprintf(tmp, 100, "destination (%u)", cfile.edt->pi.destport);
+    menuitem = gtk_menu_item_new_with_label(tmp);
+    OBJECT_SET_DATA(menuitem, "user_data", GINT_TO_POINTER(E_DECODE_DPORT));
+    gtk_menu_append(GTK_MENU(menu), menuitem);
+    gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
+
+    menuitem = gtk_menu_item_new_with_label("both");
+    OBJECT_SET_DATA(menuitem, "user_data", GINT_TO_POINTER(E_DECODE_BPORT));
+    gtk_menu_append(GTK_MENU(menu), menuitem);
+    gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
+
+    OBJECT_SET_DATA(page, E_MENU_SRCDST, menu);
+    OBJECT_SET_DATA(page, E_PAGE_TABLE, "sctp.port");
+    sctpmenu = OBJECT_GET_DATA(decode_w, "user_data");
+    gtk_option_menu_set_menu(GTK_OPTION_MENU(sctpmenu), menu);
+    OBJECT_SET_DATA(page, E_PAGE_SPORT, GINT_TO_POINTER(cfile.edt->pi.srcport));
+    OBJECT_SET_DATA(page, E_PAGE_DPORT, GINT_TO_POINTER(cfile.edt->pi.destport));  
+#if GTK_MAJOR_VERSION < 2
+    sctp_list = OBJECT_GET_DATA(decode_w, "sctp_list");
+    gtk_clist_clear(sctp_list);
+#else
+    sctp_store = OBJECT_GET_DATA(G_OBJECT(decode_w), "sctp_data");
+    gtk_list_store_clear(sctp_store);
+#endif
+    decode_sctp_list_menu_start(&list, &scrolled_window);
+    dissector_table_foreach_handle("sctp.port", decode_proto_add_to_list, list);
+    decode_list_menu_finish(list);
+}
+
+
+
+static GtkWidget *
+decode_sctp_add_port_ppid (GtkWidget *page)
+{
+    GtkWidget *format_vb, *radio_button;
+    GSList *format_grp;
+
+    format_vb = gtk_vbox_new(FALSE, 2);
+
+    radio_button = gtk_radio_button_new_with_label(NULL, "PPID");
+    format_grp = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button), TRUE);
+    SIGNAL_CONNECT(radio_button, "clicked", decode_sctp_update_ppid_menu, page);
+
+    gtk_box_pack_start(GTK_BOX(format_vb), radio_button, TRUE, TRUE, 0);
+
+    radio_button = gtk_radio_button_new_with_label(format_grp, "Port");
+    format_grp = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button));
+    SIGNAL_CONNECT(radio_button, "clicked", decode_sctp_update_srcdst_menu, page);
+
+    gtk_box_pack_start(GTK_BOX(format_vb), radio_button, TRUE, TRUE, 0);
+
+    return(format_vb);
+}
+
+
+static GtkWidget *
+decode_add_sctp_page (gchar *prompt, gchar *table_name)
+{
+    GtkWidget	*page, *label, *scrolled_window,  *radio, *vbox, *alignment, *sctpbox, *sctpmenu;
+
+    page = gtk_hbox_new(FALSE, 5);
+    OBJECT_SET_DATA(page, E_PAGE_ACTION, decode_transport);
+    OBJECT_SET_DATA(page, E_PAGE_TABLE, table_name);
+    OBJECT_SET_DATA(page, E_PAGE_TITLE, "Transport");
+
+    vbox = gtk_vbox_new(FALSE, 5);
+    radio = decode_sctp_add_port_ppid(page);
+    gtk_box_pack_start(GTK_BOX(vbox), radio, TRUE, TRUE, 0);
+
+    /* Always enabled */
+    sctpbox = gtk_hbox_new(FALSE, 5);
+    label = gtk_label_new(prompt);
+    gtk_box_pack_start(GTK_BOX(sctpbox), label, TRUE, TRUE, 0);  
+    sctpmenu = decode_add_ppid_menu(page);
+    OBJECT_SET_DATA(decode_w, "user_data", sctpmenu);
+    alignment = decode_add_pack_menu(sctpmenu);
+
+    gtk_box_pack_start(GTK_BOX(sctpbox), alignment, TRUE, TRUE, 0);
+
+    /* Conditionally enabled - only when decoding packets */
+    label = gtk_label_new("as");
+    gtk_box_pack_start(GTK_BOX(sctpbox), label, TRUE, TRUE, 0);
+    decode_dimmable = g_slist_prepend(decode_dimmable, label);
+    gtk_box_pack_start(GTK_BOX(vbox), sctpbox, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(page), vbox, TRUE, TRUE, 0);
+
+    scrolled_window = decode_add_simple_menu(page, table_name);
+    gtk_box_pack_start(GTK_BOX(page), scrolled_window, TRUE, TRUE, 0);
+    decode_dimmable = g_slist_prepend(decode_dimmable, scrolled_window);
+
+    return(page);
+}
+
+
 /*
  * This routine indicates whether we'd actually have any pages in the
  * notebook in a "Decode As" dialog box; if there wouldn't be, we
@@ -1428,6 +1813,10 @@ decode_add_notebook (GtkWidget *format_hb)
 	page = decode_add_tcpudp_page("UDP", "udp.port");
 	break;
 
+    case PT_SCTP:
+	page = decode_add_sctp_page("SCTP", "sctp.ppi");
+	break;
+
     default:
 	page = NULL;
 	break;
@@ -1437,10 +1826,18 @@ decode_add_notebook (GtkWidget *format_hb)
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
     }
 
+    if(cfile.edt->pi.dcetransporttype != -1) {
+	    page = decode_dcerpc_add_page(&cfile.edt->pi);
+	    label = gtk_label_new("DCE-RPC");
+	    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
+        OBJECT_SET_DATA(decode_w, E_PAGE_DCERPC, page);
+    }
+
     /* Select the last added page (selects first by default) */
     /* Notebook must be visible for set_page to work. */
     gtk_widget_show_all(notebook);
     gtk_notebook_set_page(GTK_NOTEBOOK(notebook), -1);
+
 }
 
 
@@ -1461,8 +1858,9 @@ decode_add_notebook (GtkWidget *format_hb)
 void
 decode_as_cb (GtkWidget * w _U_, gpointer data _U_)
 {
-    GtkWidget	*main_vb, *format_hb, *bbox, *ok_bt, *cancel_bt, *button;
+    GtkWidget	*main_vb, *format_hb, *bbox, *ok_bt, *close_bt, *button;
     GtkWidget   *button_vb, *apply_bt;
+    GtkTooltips *tooltips = gtk_tooltips_new();
 
     if (decode_w != NULL) {
 	/* There's already a "Decode As" dialog box; reactivate it. */
@@ -1492,24 +1890,39 @@ decode_as_cb (GtkWidget * w _U_, gpointer data _U_)
     SIGNAL_CONNECT(button, "clicked", decode_show_cb, decode_w);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(button_vb), button, FALSE, FALSE, 0);
+    gtk_tooltips_set_tip(tooltips, button, 
+        "Open a dialog showing the current settings.", NULL);
+
+    button = gtk_button_new_with_label("Clear");
+    SIGNAL_CONNECT(button, "clicked", decode_clear_cb, decode_w);
+    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+    gtk_box_pack_start(GTK_BOX(button_vb), button, FALSE, FALSE, 0);
+    gtk_tooltips_set_tip(tooltips, button, 
+        "Clear ALL settings.", NULL);
 
 	decode_add_notebook(format_hb);
     }
 
     /* Button row */
-    bbox = dlg_button_row_new(GTK_STOCK_OK, GTK_STOCK_APPLY, GTK_STOCK_CANCEL, NULL);
+    bbox = dlg_button_row_new(GTK_STOCK_OK, GTK_STOCK_APPLY, GTK_STOCK_CLOSE, NULL);
     gtk_box_pack_start(GTK_BOX(main_vb), bbox, FALSE, FALSE, 0);
     gtk_widget_show(bbox);
 
     ok_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_OK);
     SIGNAL_CONNECT(ok_bt, "clicked", decode_ok_cb, decode_w);
+    gtk_tooltips_set_tip(tooltips, ok_bt, 
+        "Apply current setting, close dialog and redissect packets.", NULL);
 
     apply_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_APPLY);
     SIGNAL_CONNECT(apply_bt, "clicked", decode_apply_cb, decode_w);
+    gtk_tooltips_set_tip(tooltips, apply_bt, 
+        "Apply current setting, redissect packets and keep dialog open.", NULL);
 
-    cancel_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
-    window_set_cancel_button(decode_w, cancel_bt, NULL);
-    SIGNAL_CONNECT(cancel_bt, "clicked", decode_cancel_cb, decode_w);
+    close_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
+    window_set_cancel_button(decode_w, close_bt, NULL);
+    SIGNAL_CONNECT(close_bt, "clicked", decode_close_cb, decode_w);
+    gtk_tooltips_set_tip(tooltips, close_bt, 
+        "Close the dialog, don't redissect packets.", NULL);
 
     gtk_widget_grab_default(ok_bt);
 
@@ -1519,11 +1932,3 @@ decode_as_cb (GtkWidget * w _U_, gpointer data _U_)
     gtk_widget_show_all(decode_w);
     window_present(decode_w);
 }
-
-
-/*
- * Local Variables:
- * mode:c
- * c-basic-offset: 4
- * End:
- */
