@@ -1,7 +1,7 @@
 /* fc_stat.c
  * fc_stat   2003 Ronnie Sahlberg
  *
- * $Id: fc_stat.c,v 1.34 2004/06/02 07:12:09 ulfl Exp $
+ * $Id: fc_stat.c 12128 2004-09-29 00:06:36Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,8 +39,8 @@
 #include <epan/value_string.h>
 
 #include "tap_menu.h"
-#include "../tap.h"
-#include "../packet-fc.h"
+#include <epan/tap.h>
+#include <epan/dissectors/packet-fc.h>
 #include "../register.h"
 #include "../timestats.h"
 #include "compat_macros.h"
@@ -49,7 +49,7 @@
 #include "dlg_utils.h"
 #include "../file.h"
 #include "../globals.h"
-#include "filter_prefs.h"
+#include "filter_dlg.h"
 #include "service_response_time_table.h"
 #include "gtkglobals.h"
 
