@@ -1,7 +1,7 @@
 /* file_dlg.c
  * Dialog boxes for handling files
  *
- * $Id: file_dlg.c,v 1.127 2004/06/30 07:12:25 guy Exp $
+ * $Id: file_dlg.c 12341 2004-10-18 15:14:13Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -30,14 +30,14 @@
 
 #include <gtk/gtk.h>
 
-#include "range.h"
+#include "packet-range.h"
 #include <epan/filesystem.h>
 
 #include "globals.h"
 #include "gtkglobals.h"
-#include <epan/resolv.h>
+#include <epan/addr_resolv.h>
 #include "keys.h"
-#include "filter_prefs.h"
+#include "filter_dlg.h"
 #include "ui_util.h"
 #include "alert_box.h"
 #include "simple_dialog.h"
@@ -46,11 +46,11 @@
 #include "file_dlg.h"
 #include "main.h"
 #include "compat_macros.h"
-#include "prefs.h"
+#include <epan/prefs.h>
 #include "recent.h"
 #include "color.h"
 #include "../ui_util.h"
-#include "gtk/color_filters.h"
+#include "color_filters.h"
 #include "gtk/color_dlg.h"
 #ifdef HAVE_LIBPCAP
 #include "capture_dlg.h"
@@ -78,6 +78,7 @@ static void file_color_import_ok_cb(GtkWidget *w, gpointer fs);
 static void file_color_import_destroy_cb(GtkWidget *win, gpointer user_data);
 static void file_color_export_ok_cb(GtkWidget *w, gpointer fs);
 static void file_color_export_destroy_cb(GtkWidget *win, gpointer user_data);
+static void set_file_type_list(GtkWidget *option_menu);
 
 #define E_FILE_M_RESOLVE_KEY	  "file_dlg_mac_resolve_key"
 #define E_FILE_N_RESOLVE_KEY	  "file_dlg_network_resolve_key"
@@ -107,13 +108,19 @@ static void file_color_export_destroy_cb(GtkWidget *win, gpointer user_data);
  */
 static GtkWidget *file_save_as_w;
 
+/* XXX - can we make these not be static? */
+static packet_range_t range;
+static gboolean color_marked;
+static int filetype;
+static GtkWidget *cfmark_cb;
+static GtkWidget *ft_om;
+static GtkWidget *range_tb;
 
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
 
 #define PREVIEW_STR_MAX         200
-#define PREVIEW_TIMEOUT_SECS    3
 
 static double
 secs_usecs( guint32 s, guint32 us)
@@ -123,29 +130,19 @@ secs_usecs( guint32 s, guint32 us)
 
 
 /* set a new filename for the preview widget */
-static gboolean
+static wtap *
 preview_set_filename(GtkWidget *prev, const gchar *cf_name)
 {
     GtkWidget  *label;
     wtap       *wth;
-    const struct wtap_pkthdr *phdr;
     int         err = 0;
     gchar      *err_info;
     struct stat cf_stat;
-    long        data_offset;
     gchar       string_buff[PREVIEW_STR_MAX];
-    unsigned int packet = 0;
-    double      start_time = 0;	/* seconds, with msec resolution */
-    double      stop_time = 0;	/* seconds, with msec resolution */
-    double      cur_time;
-    time_t      ti_time;
-    struct tm  *ti_tm;
-    unsigned int elapsed_time;
-    time_t      time_preview;
-    time_t      time_current;
-    gboolean    is_breaked = FALSE;
     guint64     filesize;
 
+
+    /* init preview labels */
     label = OBJECT_GET_DATA(prev, PREVIEW_FILENAME_KEY);
     gtk_label_set_text(GTK_LABEL(label), "-");
     label = OBJECT_GET_DATA(prev, PREVIEW_FORMAT_KEY);
@@ -200,11 +197,37 @@ preview_set_filename(GtkWidget *prev, const gchar *cf_name)
     label = OBJECT_GET_DATA(prev, PREVIEW_FORMAT_KEY);
     gtk_label_set_text(GTK_LABEL(label), string_buff);
 
+    return wth;
+}
+
+
+/* do a preview run on the currently selected capture file */
+static void
+preview_do(GtkWidget *prev, wtap *wth)
+{
+    GtkWidget  *label;
+    unsigned int elapsed_time;
+    time_t      time_preview;
+    time_t      time_current;
+    int         err = 0;
+    gchar      *err_info;
+    long        data_offset;
+    const struct wtap_pkthdr *phdr;
+    double      start_time = 0;	/* seconds, with msec resolution */
+    double      stop_time = 0;	/* seconds, with msec resolution */
+    double      cur_time;
+    unsigned int packets = 0;
+    gboolean    is_breaked = FALSE;
+    gchar       string_buff[PREVIEW_STR_MAX];
+    time_t      ti_time;
+    struct tm  *ti_tm;
+
+
     time(&time_preview);
     while ( (wtap_read(wth, &err, &err_info, &data_offset)) ) {
         phdr = wtap_phdr(wth);        
         cur_time = secs_usecs(phdr->ts.tv_sec, phdr->ts.tv_usec);
-        if(packet == 0) {
+        if(packets == 0) {
             start_time 	= cur_time;
             stop_time = cur_time;
         }
@@ -214,10 +237,12 @@ preview_set_filename(GtkWidget *prev, const gchar *cf_name)
         if (cur_time > stop_time){
             stop_time = cur_time;
         }
-        packet++;
-        if(packet%100) {
+
+        packets++;
+        if(packets%1000) {
+            /* do we have a timeout? */
             time(&time_current);
-            if(time_current-time_preview >= PREVIEW_TIMEOUT_SECS) {
+            if(time_current-time_preview >= (time_t) prefs.gui_fileopen_preview) {
                 is_breaked = TRUE;
                 break;
             }
@@ -225,18 +250,18 @@ preview_set_filename(GtkWidget *prev, const gchar *cf_name)
     }
 
     if(err != 0) {
-        g_snprintf(string_buff, PREVIEW_STR_MAX, "error after reading %u packets", packet);
+        g_snprintf(string_buff, PREVIEW_STR_MAX, "error after reading %u packets", packets);
         label = OBJECT_GET_DATA(prev, PREVIEW_PACKETS_KEY);
         gtk_label_set_text(GTK_LABEL(label), string_buff);
         wtap_close(wth);
-        return TRUE;
+        return;
     }
 
     /* packet count */
     if(is_breaked) {
-        g_snprintf(string_buff, PREVIEW_STR_MAX, "more than %u packets (preview timeout)", packet);
+        g_snprintf(string_buff, PREVIEW_STR_MAX, "more than %u packets (preview timeout)", packets);
     } else {
-        g_snprintf(string_buff, PREVIEW_STR_MAX, "%u", packet);
+        g_snprintf(string_buff, PREVIEW_STR_MAX, "%u", packets);
     }
     label = OBJECT_GET_DATA(prev, PREVIEW_PACKETS_KEY);
     gtk_label_set_text(GTK_LABEL(label), string_buff);
@@ -271,8 +296,6 @@ preview_set_filename(GtkWidget *prev, const gchar *cf_name)
     gtk_label_set_text(GTK_LABEL(label), string_buff);
 
     wtap_close(wth);
-
-    return TRUE;
 }
 
 #if 0
@@ -298,13 +321,14 @@ update_preview_cb (GtkFileChooser *file_chooser, gpointer data)
 #endif
 
 
-/* the text entry changed */
+/* the filename text entry changed */
 static void
 file_open_entry_changed(GtkWidget *w _U_, gpointer file_sel)
 {
     GtkWidget *prev = OBJECT_GET_DATA(file_sel, PREVIEW_TABLE_KEY);
     const gchar* cf_name;
     gboolean have_preview;
+    wtap       *wth;
 
     /* get the filename */
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
@@ -314,7 +338,8 @@ file_open_entry_changed(GtkWidget *w _U_, gpointer file_sel)
 #endif
 
     /* set the filename to the preview */
-    have_preview = preview_set_filename(prev, cf_name);
+    wth = preview_set_filename(prev, cf_name);
+    have_preview = (gboolean) wth;
 
     /* make the preview widget sensitive */
     gtk_widget_set_sensitive(prev, have_preview);
@@ -325,6 +350,10 @@ file_open_entry_changed(GtkWidget *w _U_, gpointer file_sel)
 #else
     gtk_widget_set_sensitive(GTK_FILE_SELECTION(file_sel)->ok_button, have_preview);
 #endif
+
+    /* do the actual preview */
+    if(have_preview)
+        preview_do(prev, wth);
 }
 
 
@@ -582,11 +611,11 @@ file_open_cmd(GtkWidget *w)
 static void file_open_answered_cb(gpointer dialog _U_, gint btn, gpointer data _U_)
 {
     switch(btn) {
-    case(ESD_BTN_YES):
+    case(ESD_BTN_SAVE):
         /* save file first */
         file_save_as_cmd(after_save_open_dialog, data);
         break;
-    case(ESD_BTN_NO):
+    case(ESD_BTN_DONT_SAVE):
         cf_close(&cfile);
         file_open_cmd(data);
         break;
@@ -603,7 +632,7 @@ file_open_cmd_cb(GtkWidget *widget, gpointer data _U_) {
 
   if((cfile.state != FILE_CLOSED) && !cfile.user_saved && prefs.gui_ask_unsaved) {
     /* user didn't saved his current file, ask him */
-    dialog = simple_dialog(ESD_TYPE_CONFIRMATION, ESD_BTNS_YES_NO_CANCEL,
+    dialog = simple_dialog(ESD_TYPE_CONFIRMATION, ESD_BTNS_SAVE_DONTSAVE_CANCEL,
                 PRIMARY_TEXT_START "Save capture file before opening a new one?" PRIMARY_TEXT_END "\n\n"
                 "If you open a new capture file without saving, your capture data will be discarded.");
     simple_dialog_set_cb(dialog, file_open_answered_cb, widget);
@@ -613,6 +642,7 @@ file_open_cmd_cb(GtkWidget *widget, gpointer data _U_) {
   }
 }
 
+/* user pressed "open" button */
 static void
 file_open_ok_cb(GtkWidget *w, gpointer fs) {
   gchar     *cf_name, *rfilter, *s;
@@ -709,7 +739,7 @@ file_open_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
 }
 
 /*
- * Keep a static pointer to the current "Marge Capture File" window, if
+ * Keep a static pointer to the current "Merge Capture File" window, if
  * any, so that if somebody tries to do "File:Merge" while there's already
  * an "Merge Capture File" window up, we just pop up the existing one,
  * rather than creating a new one.
@@ -720,8 +750,9 @@ static GtkWidget *file_merge_w;
 void
 file_merge_cmd(GtkWidget *w)
 {
-  GtkWidget	*main_hb, *main_vb, *filter_hbox, *filter_bt, *filter_te,
-  		*prepend_rb, *chrono_rb, *append_rb, *prev;
+  GtkWidget	*main_hb, *main_vb, *ft_hb, *ft_lb, *filter_hbox,
+		*filter_bt, *filter_te, *prepend_rb, *chrono_rb,
+		*append_rb, *prev;
 #if GTK_MAJOR_VERSION < 2
   GtkAccelGroup *accel_group;
 #endif
@@ -740,6 +771,9 @@ file_merge_cmd(GtkWidget *w)
     return;
   }
 
+  /* Default to saving all packets, in the file's current format. */
+  filetype = cfile.cd_t;
+
   file_merge_w = file_selection_new("Ethereal: Merge with Capture File",
                                    FILE_SELECTION_OPEN);
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
@@ -788,6 +822,23 @@ file_merge_cmd(GtkWidget *w)
   gtk_box_pack_start(GTK_BOX(main_hb), main_vb, FALSE, FALSE, 0);
   gtk_widget_show(main_vb);
 
+  /* File type row */
+  range_tb = NULL;
+  ft_hb = gtk_hbox_new(FALSE, 3);
+  gtk_container_add(GTK_CONTAINER(main_vb), ft_hb);
+  gtk_widget_show(ft_hb);
+
+  ft_lb = gtk_label_new("Merged output file type:");
+  gtk_box_pack_start(GTK_BOX(ft_hb), ft_lb, FALSE, FALSE, 0);
+  gtk_widget_show(ft_lb);
+
+  ft_om = gtk_option_menu_new();
+
+  /* Generate the list of file types we can save. */
+  set_file_type_list(ft_om);
+  gtk_box_pack_start(GTK_BOX(ft_hb), ft_om, FALSE, FALSE, 0);
+  gtk_widget_show(ft_om);
+
   filter_hbox = gtk_hbox_new(FALSE, 1);
   gtk_container_border_width(GTK_CONTAINER(filter_hbox), 0);
   gtk_box_pack_start(GTK_BOX(main_vb), filter_hbox, FALSE, FALSE, 0);
@@ -973,19 +1024,19 @@ file_merge_ok_cb(GtkWidget *w, gpointer fs) {
       /* chonological order */
       in_filenames[0] = cfile.filename;
       in_filenames[1] = cf_name;
-      merge_ok = merge_n_files(out_fd, 2, in_filenames, FALSE, &err);
+      merge_ok = merge_n_files(out_fd, 2, in_filenames, filetype, FALSE, &err);
   } else {
       rb = OBJECT_GET_DATA(w, E_MERGE_PREPEND_KEY);
       if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (rb))) {
           /* prepend file */
           in_filenames[0] = cfile.filename;
           in_filenames[1] = cf_name;
-          merge_ok = merge_n_files(out_fd, 2, in_filenames, TRUE, &err);
+          merge_ok = merge_n_files(out_fd, 2, in_filenames, filetype, TRUE, &err);
       } else {
           /* append file */
           in_filenames[0] = cf_name;
           in_filenames[1] = cfile.filename;
-          merge_ok = merge_n_files(out_fd, 2, in_filenames, TRUE, &err);
+          merge_ok = merge_n_files(out_fd, 2, in_filenames, filetype, TRUE, &err);
       }
   }
 
@@ -1059,11 +1110,11 @@ file_merge_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
 void file_close_answered_cb(gpointer dialog _U_, gint btn, gpointer data _U_)
 {
     switch(btn) {
-    case(ESD_BTN_YES):
+    case(ESD_BTN_SAVE):
         /* save file first */
         file_save_as_cmd(after_save_close_file, NULL);
         break;
-    case(ESD_BTN_NO):
+    case(ESD_BTN_DONT_SAVE):
         cf_close(&cfile);
         break;
     case(ESD_BTN_CANCEL):
@@ -1080,7 +1131,7 @@ file_close_cmd_cb(GtkWidget *widget _U_, gpointer data _U_) {
 
   if((cfile.state != FILE_CLOSED) && !cfile.user_saved && prefs.gui_ask_unsaved) {
     /* user didn't saved his current file, ask him */
-    dialog = simple_dialog(ESD_TYPE_CONFIRMATION, ESD_BTNS_YES_NO_CANCEL,
+    dialog = simple_dialog(ESD_TYPE_CONFIRMATION, ESD_BTNS_SAVE_DONTSAVE_CANCEL,
                 PRIMARY_TEXT_START "Save capture file before closing it?" PRIMARY_TEXT_END "\n\n"
                 "If you close without saving, your capture data will be discarded.");
 
@@ -1101,14 +1152,6 @@ file_save_cmd_cb(GtkWidget *w, gpointer data) {
   file_save_as_cmd_cb(w, data);
 }
 
-/* XXX - can we make these not be static? */
-static packet_range_t range;
-static gboolean color_marked;
-static int filetype;
-static GtkWidget *cfmark_cb;
-static GtkWidget *ft_om;
-static GtkWidget *range_tb;
-
 static gboolean
 can_save_with_wiretap(int ft)
 {
@@ -1181,7 +1224,8 @@ select_file_type_cb(GtkWidget *w _U_, gpointer data)
   if (filetype != new_filetype) {
     /* We can select only the filtered or marked packets to be saved if we can
        use Wiretap to save the file. */
-    range_set_displayed_sensitive(range_tb, can_save_with_wiretap(new_filetype));
+    if (range_tb != NULL)
+       range_set_displayed_sensitive(range_tb, can_save_with_wiretap(new_filetype));
     filetype = new_filetype;
     file_set_save_marked_sensitive();
   }
@@ -1205,7 +1249,7 @@ file_set_save_marked_sensitive(void)
 	
   /* We can request that only the marked packets be saved only if we
      can use Wiretap to save the file and if there *are* marked packets. */
-  if (can_save_with_wiretap(filetype) && cfile.marked_count != 0) {
+  if (can_save_with_wiretap(filetype) && cfile.marked_count > 0) {
     range_set_marked_sensitive(range_tb, TRUE);
   }
   else {
@@ -1363,6 +1407,24 @@ file_save_as_ok_cb(GtkWidget *w _U_, gpointer fs) {
         return;
   }
 
+  /* Check whether the range is valid. */
+  if (!range_check_validity(&range)) {
+    /* The range isn't valid; don't dismiss the open dialog box,
+       just leave it around so that the user can, after they
+       dismiss the alert box popped up for the error, try again. */
+    g_free(cf_name);
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
+     * as this will prevent the user from closing the now existing error
+     * message, simply close the dialog (this is the best we can do here). */
+    if (file_save_as_w)
+      window_destroy(GTK_WIDGET (fs));
+#else
+    gtk_widget_show(GTK_WIDGET (fs));
+#endif
+    return;
+  }
+
   /* don't show the dialog while saving */
   gtk_widget_hide(GTK_WIDGET (fs));
 
@@ -1373,7 +1435,15 @@ file_save_as_ok_cb(GtkWidget *w _U_, gpointer fs) {
        just leave it around so that the user can, after they
        dismiss the alert box popped up for the error, try again. */
     g_free(cf_name);
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
+     * as this will prevent the user from closing the now existing error
+     * message, simply close the dialog (this is the best we can do here). */
+    if (file_save_as_w)
+      window_destroy(GTK_WIDGET (fs));
+#else
     gtk_widget_show(GTK_WIDGET (fs));
+#endif
     return;
   }
 
@@ -1438,52 +1508,7 @@ file_save_as_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
 /* Reload a file using the current read and display filters */
 void
 file_reload_cmd_cb(GtkWidget *w _U_, gpointer data _U_) {
-  gchar *filename;
-  gboolean is_tempfile;
-
-  /* If the file could be opened, "cf_open()" calls "cf_close()"
-     to get rid of state for the old capture file before filling in state
-     for the new capture file.  "cf_close()" will remove the file if
-     it's a temporary file; we don't want that to happen (for one thing,
-     it'd prevent subsequent reopens from working).  Remember whether it's
-     a temporary file, mark it as not being a temporary file, and then
-     reopen it as the type of file it was.
-
-     Also, "cf_close()" will free "cfile.filename", so we must make
-     a copy of it first. */
-  filename = g_strdup(cfile.filename);
-  is_tempfile = cfile.is_tempfile;
-  cfile.is_tempfile = FALSE;
-  if (cf_open(filename, is_tempfile, &cfile) == 0) {
-    switch (cf_read(&cfile)) {
-
-    case READ_SUCCESS:
-    case READ_ERROR:
-      /* Just because we got an error, that doesn't mean we were unable
-         to read any of the file; we handle what we could get from the
-         file. */
-      break;
-
-    case READ_ABORTED:
-      /* The user bailed out of re-reading the capture file; the
-         capture file has been closed - just free the capture file name
-         string and return (without changing the last containing
-         directory). */
-      g_free(filename);
-      return;
-    }
-  } else {
-    /* The open failed, so "cfile.is_tempfile" wasn't set to "is_tempfile".
-       Instead, the file was left open, so we should restore "cfile.is_tempfile"
-       ourselves.
-
-       XXX - change the menu?  Presumably "cf_open()" will do that;
-       make sure it does! */
-    cfile.is_tempfile = is_tempfile;
-  }
-  /* "cf_open()" made a copy of the file name we handed it, so
-     we should free up our copy. */
-  g_free(filename);
+  cf_reload();
 }
 
 /******************** Color Filters *********************************/
