@@ -5,7 +5,7 @@
  *
  * MUCH code modified from service_response_time_table.c.
  *
- * $Id: gsm_a_stat.c,v 1.18 2004/05/23 23:24:05 ulfl Exp $
+ * $Id: gsm_a_stat.c 12128 2004-09-29 00:06:36Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -45,12 +45,12 @@
 #include "image/clist_descend.xpm"
 #include "simple_dialog.h"
 #include "dlg_utils.h"
-#include "tap.h"
+#include <epan/tap.h>
 #include "../register.h"
-#include "packet-bssap.h"
-#include "packet-gsm_a.h"
+#include <epan/dissectors/packet-bssap.h>
+#include <epan/dissectors/packet-gsm_a.h>
 #include "../globals.h"
-#include "filter_prefs.h"
+#include "filter_dlg.h"
 #include "compat_macros.h"
 #include "ui_util.h"
 
