@@ -1,7 +1,7 @@
 /* gui_prefs.c
  * Dialog box for GUI preferences
  *
- * $Id: gui_prefs.c,v 1.75 2004/06/29 17:10:53 ulfl Exp $
+ * $Id: gui_prefs.c 12115 2004-09-27 22:55:15Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,7 +35,7 @@
 #include "gtkglobals.h"
 #include "help_dlg.h"
 #include "supported_protos_dlg.h"
-#include "prefs.h"
+#include <epan/prefs.h>
 #include "prefs_dlg.h"
 #include "ui_util.h"
 #include "simple_dialog.h"
@@ -52,6 +52,7 @@
 
 static gint fetch_enum_value(gpointer control, const enum_val_t *enumvals);
 static gint fileopen_dir_changed_cb(GtkWidget *myentry _U_, GdkEvent *event, gpointer parent_w);
+static gint fileopen_preview_changed_cb(GtkWidget *myentry _U_, GdkEvent *event, gpointer parent_w);
 static void fileopen_selected_cb(GtkWidget *mybutton_rb _U_, gpointer parent_w);
 static gint recent_files_count_changed_cb(GtkWidget *recent_files_entry _U_, 
 					  GdkEvent *event _U_, gpointer parent_w);
@@ -73,10 +74,12 @@ static gint recent_files_count_changed_cb(GtkWidget *recent_files_entry _U_,
 
 #define GUI_CONSOLE_OPEN_KEY "console_open"
 #define GUI_FILEOPEN_KEY	"fileopen_behavior"
+#define GUI_FILEOPEN_PREVIEW_KEY "fileopen_preview_timeout"
 #define GUI_RECENT_FILES_COUNT_KEY "recent_files_count"
 #define GUI_FILEOPEN_DIR_KEY	"fileopen_directory"
 #define GUI_ASK_UNSAVED_KEY     "ask_unsaved"
 #define GUI_WEBBROWSER_KEY	    "webbrowser"
+#define GUI_FIND_WRAP_KEY       "find_wrap"
 
 #define GUI_TOOLBAR_STYLE_KEY	"toolbar_style"
 
@@ -128,6 +131,7 @@ static const enum_val_t highlight_style_vals[] = {
 	{ NULL,    NULL,       0 }
 };
 
+
 static const enum_val_t toolbar_style_vals[] = {
   	{ "ICONS", "Icons only",     TB_STYLE_ICONS },
   	{ "TEXT",  "Text only",      TB_STYLE_TEXT },
@@ -165,6 +169,9 @@ static GtkWidget *font_browse_w;
 /* Used to contain the string from the Recent Files Count Max pref item */
 static char recent_files_count_max_str[128] = "";
 
+/* Used to contain the string from the Open File preview timeout pref item */
+static char open_file_preview_str[128] = "";
+
 #if GTK_MAJOR_VERSION < 2
 #define GUI_TABLE_ROWS 10
 #else
@@ -180,9 +187,10 @@ gui_prefs_show(void)
 #ifdef _WIN32
 	GtkWidget *console_open_om;
 #endif
-	GtkWidget *fileopen_rb, *fileopen_dir_te, *toolbar_style_om;
+	GtkWidget *fileopen_rb, *fileopen_dir_te, *fileopen_preview_te;
+	GtkWidget *toolbar_style_om;
     GtkWidget *filter_toolbar_placement_om;
-	GtkWidget *recent_files_count_max_te, *ask_unsaved_cb;
+	GtkWidget *recent_files_count_max_te, *ask_unsaved_cb, *find_wrap_cb;
     GtkWidget *webbrowser_te;
 	GtkWidget *save_position_cb, *save_size_cb, *save_maximized_cb;
 #if GTK_MAJOR_VERSION < 2
@@ -306,6 +314,14 @@ gui_prefs_show(void)
 	SIGNAL_CONNECT(fileopen_dir_te, "focus-out-event",
 	    fileopen_dir_changed_cb, main_vb);
 
+	/* File Open dialog preview timeout */
+	fileopen_preview_te = create_preference_entry(main_tb, pos++,
+	    "\"File Open\" preview timeout:", "Timeout, until preview gives up scanning the capture file content.", open_file_preview_str);
+	g_snprintf(current_val_str, 128, "%d", prefs.gui_fileopen_preview);
+	gtk_entry_set_text(GTK_ENTRY(fileopen_preview_te), current_val_str);
+	OBJECT_SET_DATA(main_vb, GUI_FILEOPEN_PREVIEW_KEY, fileopen_preview_te);
+	SIGNAL_CONNECT(fileopen_preview_te, "focus_out_event", fileopen_preview_changed_cb, main_vb);
+
 	/* Number of entries in the recent_files list ... */
 	recent_files_count_max_te = create_preference_entry(main_tb, pos++,
 	    "\"Open Recent\" max. list entries:", "Maximum number of recent files", recent_files_count_max_str);
@@ -321,6 +337,11 @@ gui_prefs_show(void)
 	    "Ask for unsaved capture files:", NULL, prefs.gui_ask_unsaved);
 	OBJECT_SET_DATA(main_vb, GUI_ASK_UNSAVED_KEY, ask_unsaved_cb);
 
+    /* do we want to wrap when searching for data? */
+	find_wrap_cb = create_preference_check_button(main_tb, pos++,
+	    "Wrap to end/beginning of file during a find:", NULL, prefs.gui_find_wrap);
+	OBJECT_SET_DATA(main_vb, GUI_FIND_WRAP_KEY, find_wrap_cb);
+
 	/* Webbrowser */
     if(browser_needs_pref()) {
 	    webbrowser_te = create_preference_entry(main_tb, pos++, 
@@ -365,12 +386,14 @@ font_fetch(void)
 		return FALSE;
 	}
 
-    if(user_font_test(font_name)) {
-    	new_font_name = font_name;
-        return TRUE;
-    }
-
-    g_free(font_name);
+	if (!user_font_test(font_name)) {
+		/* The font isn't usable; "user_font_test()" has already
+		   told the user why.  Don't tear down the font selection
+		   dialog. */
+		g_free(font_name);
+		return FALSE;
+	}
+	new_font_name = font_name;
 	return TRUE;
 }
 
@@ -430,6 +453,9 @@ gui_prefs_fetch(GtkWidget *w)
     prefs.gui_ask_unsaved = 
 	    gtk_toggle_button_get_active(OBJECT_GET_DATA(w, GUI_ASK_UNSAVED_KEY));
 
+    prefs.gui_find_wrap = 
+	    gtk_toggle_button_get_active(OBJECT_GET_DATA(w, GUI_FIND_WRAP_KEY));
+    
     if(browser_needs_pref()) {
 		g_free(prefs.gui_webbrowser);
 	    prefs.gui_webbrowser = g_strdup(gtk_entry_get_text(
@@ -542,7 +568,32 @@ recent_files_count_changed_cb(GtkWidget *recent_files_entry _U_,
 
     /* We really should pop up a nasty dialog box if newval <= 0 */
 
-    return TRUE;
+    return FALSE;
+}
+
+static gint
+fileopen_preview_changed_cb(GtkWidget *recent_files_entry _U_, 
+			      GdkEvent *event _U_, gpointer parent_w)
+{
+    GtkWidget	*fileopen_preview_te;
+    guint newval;
+    
+    fileopen_preview_te = (GtkWidget *)OBJECT_GET_DATA(parent_w, GUI_FILEOPEN_PREVIEW_KEY);
+
+    /*
+     * Now, just convert the string to a number and store it in the prefs
+     * filed ...
+     */
+
+    newval = strtol(gtk_entry_get_text (GTK_ENTRY(fileopen_preview_te)), NULL, 10);
+
+    if (newval > 0) {
+      prefs.gui_fileopen_preview = newval;
+    }
+
+    /* We really should pop up a nasty dialog box if newval <= 0 */
+
+    return FALSE;
 }
 
 static gint
