@@ -1,7 +1,7 @@
 /* http_stat.c
  * http_stat   2003 Jean-Michel FAYARD
  *
- * $Id: http_stat.c,v 1.30 2004/05/23 23:24:06 ulfl Exp $
+ * $Id: http_stat.c 12128 2004-09-29 00:06:36Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -37,9 +37,9 @@
 #include "simple_dialog.h"
 #include "ui_util.h"
 #include "dlg_utils.h"
-#include "tap.h"
+#include <epan/tap.h>
 #include "../register.h"
-#include "../packet-http.h"
+#include <epan/dissectors/packet-http.h>
 #include "../globals.h"
 #include "compat_macros.h"
 #include "../tap_dfilter_dlg.h"
