@@ -1,7 +1,7 @@
 /* io_stat.c
  * io_stat   2002 Ronnie Sahlberg
  *
- * $Id: io_stat.c,v 1.76 2004/05/23 23:24:06 ulfl Exp $
+ * $Id: io_stat.c 12128 2004-09-29 00:06:36Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -43,7 +43,7 @@
 #include "gtkglobals.h"
 #include "ui_util.h"
 #include "tap_menu.h"
-#include "../tap.h"
+#include <epan/tap.h>
 #include "../register.h"
 #include "alert_box.h"
 #include "simple_dialog.h"
@@ -51,10 +51,7 @@
 #include "../color.h"
 #include "compat_macros.h"
 #include "dlg_utils.h"
-
-/* filter prefs dialog */
-#include "filter_prefs.h"
-/* filter prefs dialog */
+#include "filter_dlg.h"
 
 void protect_thread_critical_region(void);
 void unprotect_thread_critical_region(void);
@@ -1515,6 +1512,13 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 
 	field=(char *)gtk_entry_get_text(GTK_ENTRY(gio->calc_field));
 
+	/* this graph is not active, just update display and redraw */
+	if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gio->display_button))){
+		disable_graph(gio);
+		io_stat_redraw(gio->io);
+		return 0;
+	}
+
 	/* first check if the field string is valid */
 	if(gio->io->count_type==COUNT_TYPE_ADVANCED){
 		/* warn and bail out if there was no field specified */
@@ -1618,13 +1622,6 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 	if (dfilter != NULL)
 		dfilter_free(dfilter);
 
-	/* this graph is not active, just update display and redraw */
-	if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gio->display_button))){
-		disable_graph(gio);
-		io_stat_redraw(gio->io);
-		return 0;
-	}
-
 	/* ok, we have a valid filter and the graph is active.
 	   first just try to delete any previous settings and then apply
 	   the new ones.
