@@ -1,7 +1,7 @@
 /* sip_stat.c
  * sip_stat   2004 Martin Mathieson
  *
- * $Id: sip_stat.c,v 1.6 2004/05/23 23:24:06 ulfl Exp $
+ * $Id: sip_stat.c 12128 2004-09-29 00:06:36Z guy $
  * Copied from http_stat.c
  *
  * Ethereal - Network traffic analyzer
@@ -38,9 +38,9 @@
 #include "simple_dialog.h"
 #include "ui_util.h"
 #include "dlg_utils.h"
-#include "tap.h"
+#include <epan/tap.h>
 #include "../register.h"
-#include "../packet-sip.h"
+#include <epan/dissectors/packet-sip.h>
 #include "../globals.h"
 #include "compat_macros.h"
 #include "../tap_dfilter_dlg.h"
