@@ -1,7 +1,7 @@
 /* wsp_stat.c
  * wsp_stat   2003 Jean-Michel FAYARD
  *
- * $Id: wsp_stat.c,v 1.27 2004/05/23 23:24:07 ulfl Exp $
+ * $Id: wsp_stat.c 12128 2004-09-29 00:06:36Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -38,11 +38,11 @@
 #include "simple_dialog.h"
 #include "ui_util.h"
 #include "dlg_utils.h"
-#include "tap.h"
+#include <epan/tap.h>
 #include "../register.h"
 #include "../globals.h"
 #include "compat_macros.h"
-#include "../packet-wsp.h"
+#include <epan/dissectors/packet-wsp.h>
 #include "../tap_dfilter_dlg.h"
 #include "tap_dfilter_dlg.h"
 
