@@ -1,7 +1,7 @@
 /* packet-rtnet.c
  * Routines for RTnet packet disassembly
  *
- * $Id: packet-rtnet.c,v 1.7 2004/02/18 05:33:35 guy Exp $
+ * $Id: packet-rtnet.c 11615 2004-08-06 19:57:49Z guy $
  *
  * Copyright (c) 2003 by Erwin Rol <erwin@erwinrol.com>
  *
@@ -41,7 +41,7 @@
 #include <time.h>
 #include <string.h>
 #include <epan/packet.h>
-#include <epan/resolv.h>
+#include <epan/addr_resolv.h>
 #include "etypes.h"
 #include <epan/strutil.h>
 
