@@ -1,7 +1,7 @@
 /* print.c
  * Routines for printing packet analysis trees.
  *
- * $Id: print.c,v 1.83 2004/07/05 16:39:20 ulfl Exp $
+ * $Id: print.c 11958 2004-09-10 22:59:37Z guy $
  *
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -24,6 +24,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+/* With MSVC and a libethereal.dll this file needs to import some variables 
+   in a special way. Therefore _NEED_VAR_IMPORT_ is defined. */  
+#define _NEED_VAR_IMPORT_
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -36,38 +40,48 @@
 #include <epan/tvbuff.h>
 #include <epan/packet.h>
 
-#include "range.h"
+#include "packet-range.h"
 #include "print.h"
 #include "ps.h"
-#include "util.h"
-#include "packet-data.h"
-#include "packet-frame.h"
+#include <epan/charsets.h>
+#include <epan/dissectors/packet-data.h>
+#include <epan/dissectors/packet-frame.h>
 
 #define PDML_VERSION "0"
 #define PSML_VERSION "0"
 
+typedef struct {
+	int			level;
+	print_stream_t		*stream;
+	gboolean		success;
+	GSList		 	*src_list;
+	print_dissections_e	print_dissections;
+	gboolean		print_hex_for_data;
+	char_enc		encoding;
+	epan_dissect_t		*edt;
+} print_data;
+
+typedef struct {
+	int			level;
+	FILE			*fh;
+	GSList		 	*src_list;
+	epan_dissect_t		*edt;
+} write_pdml_data;
+
 static void proto_tree_print_node(proto_node *node, gpointer data);
-static void proto_tree_print_node_pdml(proto_node *node, gpointer data);
-static void print_hex_data_buffer(FILE *fh, register const guchar *cp,
-    register guint length, char_enc encoding, print_format_e format);
+static void proto_tree_write_node_pdml(proto_node *node, gpointer data);
+static const guint8 *get_field_data(GSList *src_list, field_info *fi);
+static void write_pdml_field_hex_value(write_pdml_data *pdata, field_info *fi);
+static gboolean print_hex_data_buffer(print_stream_t *stream, const guchar *cp,
+    guint length, char_enc encoding);
 static void ps_clean_string(unsigned char *out, const unsigned char *in,
 			int outbuf_size);
 static void print_escaped_xml(FILE *fh, char *unescaped_string);
 
-typedef struct {
-	int		            level;
-	FILE		        *fh;
-	GSList		        *src_list;
-	print_dissections_e print_dissections;
-	gboolean	        print_hex_for_data;
-	char_enc	        encoding;
-	gint		        format;
-	epan_dissect_t	    *edt;
-} print_data;
+static void print_pdml_geninfo(proto_tree *tree, FILE *fh);
 
-static void print_pdml_geninfo(proto_tree *tree, print_data *pdata);
-
-FILE *open_print_dest(int to_file, const char *dest)
+static FILE *
+open_print_dest(int to_file, const char *dest)
 {
 	FILE	*fh;
 
@@ -80,7 +94,8 @@ FILE *open_print_dest(int to_file, const char *dest)
 	return fh;
 }
 
-gboolean close_print_dest(int to_file, FILE *fh)
+static gboolean
+close_print_dest(int to_file, FILE *fh)
 {
 	/* Close the file or command */
 	if (to_file)
@@ -89,115 +104,32 @@ gboolean close_print_dest(int to_file, FILE *fh)
 		return (pclose(fh) == 0);
 }
 
+#define MAX_PS_LINE_LENGTH 256
 
-void
+gboolean
 proto_tree_print(print_args_t *print_args, epan_dissect_t *edt,
-    FILE *fh)
+    print_stream_t *stream)
 {
 	print_data data;
-	gint	i;
 
 	/* Create the output */
 	data.level = 0;
-	data.fh = fh;
+	data.stream = stream;
+	data.success = TRUE;
 	data.src_list = edt->pi.data_src;
 	data.encoding = edt->pi.fd->flags.encoding;
 	data.print_dissections = print_args->print_dissections;
+	/* If we're printing the entire packet in hex, don't
+	   print uninterpreted data fields in hex as well. */
 	data.print_hex_for_data = !print_args->print_hex;
-	    /* If we're printing the entire packet in hex, don't
-	       print uninterpreted data fields in hex as well. */
-	data.format = print_args->format;
 	data.edt = edt;
 
-    switch(data.format) {
-    case(PR_FMT_TEXT):  /* fall through */
-    case(PR_FMT_PS):
-		proto_tree_children_foreach(edt->tree, proto_tree_print_node, &data);
-        break;
-    case(PR_FMT_PDML):
-		fprintf(fh, "<packet>\n");
-
-		/* Print a "geninfo" protocol as required by PDML */
-		print_pdml_geninfo(edt->tree, &data);
-
-		proto_tree_children_foreach(edt->tree, proto_tree_print_node_pdml, &data);
-
-		fprintf(fh, "</packet>\n\n");
-        break;
-    case(PR_FMT_PSML):
-        /* if this is the first packet, we have to create the PSML structure output */
-        if(edt->pi.fd->num == 1) {
-            fprintf(fh, "<structure>\n");
-
-            for(i=0; i < edt->pi.cinfo->num_cols; i++) {
-                fprintf(fh, "<section>");
-                print_escaped_xml(fh, edt->pi.cinfo->col_title[i]);
-                fprintf(fh, "</section>\n");
-            }
-
-            fprintf(fh, "</structure>\n\n");
-        }
-
-		fprintf(fh, "<packet>\n");
-
-        for(i=0; i < edt->pi.cinfo->num_cols; i++) {
-            fprintf(fh, "<section>");
-            print_escaped_xml(fh, edt->pi.cinfo->col_data[i]);
-            fprintf(fh, "</section>\n");
-        }
-
-		fprintf(fh, "</packet>\n\n");
-        break;
-    default:
-        g_assert_not_reached();
-	}
-}
-
-/*
- * Find the data source for a specified field, and return a pointer
- * to the data in it. Returns NULL if the data is out of bounds.
- */
-static const guint8 *
-get_field_data(GSList *src_list, field_info *fi)
-{
-	GSList *src_le;
-	data_source *src;
-	tvbuff_t *src_tvb;
-	gint length, tvbuff_length;
-
-	for (src_le = src_list; src_le != NULL; src_le = src_le->next) {
-		src = src_le->data;
-		src_tvb = src->tvb;
-		if (fi->ds_tvb == src_tvb) {
-			/*
-			 * Found it.
-			 *
-			 * XXX - a field can have a length that runs past
-			 * the end of the tvbuff.  Ideally, that should
-			 * be fixed when adding an item to the protocol
-			 * tree, but checking the length when doing
-			 * that could be expensive.  Until we fix that,
-			 * we'll do the check here.
-			 */
-			tvbuff_length = tvb_length_remaining(src_tvb,
-			    fi->start);
-			if (tvbuff_length < 0) {
-				return NULL;
-			}
-			length = fi->length;
-			if (length > tvbuff_length)
-				length = tvbuff_length;
-			return tvb_get_ptr(src_tvb, fi->start, length);
-		}
-	}
-	g_assert_not_reached();
-	return NULL;	/* not found */
+	proto_tree_children_foreach(edt->tree, proto_tree_print_node, &data);
+	return data.success;
 }
 
 #define MAX_INDENT	160
 
-#define MAX_PS_LINE_LENGTH 256
-
 /* Print a tree's data, and any child nodes. */
 static
 void proto_tree_print_node(proto_node *node, gpointer data)
@@ -212,7 +144,11 @@ void proto_tree_print_node(proto_node *node, gpointer data)
 	if (PROTO_ITEM_IS_HIDDEN(node))
 		return;
 
-    /* was a free format label produced? */
+	/* Give up if we've already gotten an error. */
+	if (!pdata->success)
+		return;
+
+	/* was a free format label produced? */
 	if (fi->rep) {
 		label_ptr = fi->rep->representation;
 	}
@@ -221,7 +157,10 @@ void proto_tree_print_node(proto_node *node, gpointer data)
 		proto_item_fill_label(fi, label_str);
 	}
 
-	print_line(pdata->fh, pdata->level, pdata->format, label_ptr);
+	if (!print_line(pdata->stream, pdata->level, label_ptr)) {
+		pdata->success = FALSE;
+		return;
+	}
 
 	/* If it's uninterpreted data, dump it (unless our caller will
 	   be printing the entire packet in hex). */
@@ -231,8 +170,11 @@ void proto_tree_print_node(proto_node *node, gpointer data)
 		 */
 		pd = get_field_data(pdata->src_list, fi);
 		if (pd) {
-			print_hex_data_buffer(pdata->fh, pd, fi->length,
-			    pdata->encoding, pdata->format);
+			if (!print_hex_data_buffer(pdata->stream, pd,
+			    fi->length, pdata->encoding)) {
+				pdata->success = FALSE;
+				return;
+			}
 		}
 	}
 
@@ -242,76 +184,54 @@ void proto_tree_print_node(proto_node *node, gpointer data)
 	g_assert(fi->tree_type >= -1 && fi->tree_type < num_tree_types);
 	if (pdata->print_dissections == print_dissections_expanded ||
 	    (pdata->print_dissections == print_dissections_as_displayed &&
-        fi->tree_type >= 0 && tree_is_expanded[fi->tree_type])) {
+		fi->tree_type >= 0 && tree_is_expanded[fi->tree_type])) {
 		if (node->first_child != NULL) {
 			pdata->level++;
 			proto_tree_children_foreach(node,
 				proto_tree_print_node, pdata);
 			pdata->level--;
+			if (!pdata->success)
+				return;
 		}
 	}
 }
 
-/* Print a string, escaping out certain characters that need to
- * escaped out for XML. */
-static void
-print_escaped_xml(FILE *fh, char *unescaped_string)
+void
+write_pdml_preamble(FILE *fh)
 {
-	unsigned char *p;
-
-	for (p = unescaped_string; *p != '\0'; p++) {
-		switch (*p) {
-			case '&':
-				fputs("&amp;", fh);
-				break;
-			case '<':
-				fputs("&lt;", fh);
-				break;
-			case '>':
-				fputs("&gt;", fh);
-				break;
-			case '"':
-				fputs("&quot;", fh);
-				break;
-			case '\'':
-				fputs("&apos;", fh);
-				break;
-			default:
-				fputc(*p, fh);
-		}
-	}
+	fputs("<?xml version=\"1.0\"?>\n", fh);
+	fputs("<pdml version=\"" PDML_VERSION "\" ", fh);
+	fprintf(fh, "creator=\"%s/%s\">\n", PACKAGE, VERSION);
 }
 
-static void
-print_field_hex_value(print_data *pdata, field_info *fi)
+void
+proto_tree_write_pdml(epan_dissect_t *edt, FILE *fh)
 {
-	int i;
-	const guint8 *pd;
+	write_pdml_data data;
 
+	/* Create the output */
+	data.level = 0;
+	data.fh = fh;
+	data.src_list = edt->pi.data_src;
+	data.edt = edt;
 
-    if(fi->length > tvb_length_remaining(fi->ds_tvb, fi->start)) {
-        fprintf(pdata->fh, "field length invalid!");
-        return;
-    }
+	fprintf(fh, "<packet>\n");
 
-	/* Find the data for this field. */
-	pd = get_field_data(pdata->src_list, fi);
+	/* Print a "geninfo" protocol as required by PDML */
+	print_pdml_geninfo(edt->tree, fh);
 
-	if (pd) {
-		/* Print a simple hex dump */
-		for (i = 0 ; i < fi->length; i++) {
-			fprintf(pdata->fh, "%02x", pd[i]);
-		}
-	}
+	proto_tree_children_foreach(edt->tree, proto_tree_write_node_pdml,
+	    &data);
+
+	fprintf(fh, "</packet>\n\n");
 }
 
-
-/* Print a tree's data, and any child nodes, as PDML */
+/* Write out a tree's data, and any child nodes, as PDML */
 static void
-proto_tree_print_node_pdml(proto_node *node, gpointer data)
+proto_tree_write_node_pdml(proto_node *node, gpointer data)
 {
 	field_info	*fi = PITEM_FINFO(node);
-	print_data	*pdata = (print_data*) data;
+	write_pdml_data	*pdata = (write_pdml_data*) data;
 	gchar		*label_ptr;
 	gchar		label_str[ITEM_LABEL_LENGTH];
 	char		*dfilter_string;
@@ -339,7 +259,7 @@ proto_tree_print_node_pdml(proto_node *node, gpointer data)
 		fprintf(pdata->fh, "\" pos=\"%d", fi->start);
 
 		fputs("\" value=\"", pdata->fh);
-		print_field_hex_value(pdata, fi);
+		write_pdml_field_hex_value(pdata, fi);
 
 		if (node->first_child != NULL) {
 			fputs("\">\n", pdata->fh);
@@ -354,7 +274,7 @@ proto_tree_print_node_pdml(proto_node *node, gpointer data)
 
 		fputs("<field name=\"data\" value=\"", pdata->fh);
 
-		print_field_hex_value(pdata, fi);
+		write_pdml_field_hex_value(pdata, fi);
 
 		fputs("\"/>\n", pdata->fh);
 
@@ -370,19 +290,19 @@ proto_tree_print_node_pdml(proto_node *node, gpointer data)
 		print_escaped_xml(pdata->fh, fi->hfinfo->abbrev);
 
 #if 0
-        /* PDML spec, see: 
-         * http://analyzer.polito.it/30alpha/docs/dissectors/PDMLSpec.htm
-         *
-         * the show fields contains things in 'human readable' format
-         * showname: contains only the name of the field
-         * show: contains only the data of the field
-         * showdtl: contains additional details of the field data
-         * showmap: contains mappings of the field data (e.g. the hostname to an IP address)
-         *
-         * XXX - the showname shouldn't contain the field data itself 
-         * (like it's contained in the fi->rep->representation). 
-         * Unfortunately, we don't have the field data representation for 
-         * all fields, so this isn't currently possible */
+	/* PDML spec, see: 
+	 * http://analyzer.polito.it/30alpha/docs/dissectors/PDMLSpec.htm
+	 *
+	 * the show fields contains things in 'human readable' format
+	 * showname: contains only the name of the field
+	 * show: contains only the data of the field
+	 * showdtl: contains additional details of the field data
+	 * showmap: contains mappings of the field data (e.g. the hostname to an IP address)
+	 *
+	 * XXX - the showname shouldn't contain the field data itself 
+	 * (like it's contained in the fi->rep->representation). 
+	 * Unfortunately, we don't have the field data representation for 
+	 * all fields, so this isn't currently possible */
 		fputs("\" showname=\"", pdata->fh);
 		print_escaped_xml(pdata->fh, fi->hfinfo->name);
 #endif
@@ -398,9 +318,8 @@ proto_tree_print_node_pdml(proto_node *node, gpointer data)
 			print_escaped_xml(pdata->fh, label_ptr);
 		}
 
-        if(PROTO_ITEM_IS_HIDDEN(node)) {
-    		fprintf(pdata->fh, "\" hide=\"yes");
-        }
+		if (PROTO_ITEM_IS_HIDDEN(node))
+			fprintf(pdata->fh, "\" hide=\"yes");
 
 		fprintf(pdata->fh, "\" size=\"%d", fi->length);
 		fprintf(pdata->fh, "\" pos=\"%d", fi->start);
@@ -431,7 +350,7 @@ proto_tree_print_node_pdml(proto_node *node, gpointer data)
 			}
 			if (fi->length > 0) {
 				fputs("\" value=\"", pdata->fh);
-				print_field_hex_value(pdata, fi);
+				write_pdml_field_hex_value(pdata, fi);
 			}
 		}
 
@@ -450,7 +369,7 @@ proto_tree_print_node_pdml(proto_node *node, gpointer data)
 	if (node->first_child != NULL) {
 		pdata->level++;
 		proto_tree_children_foreach(node,
-				proto_tree_print_node_pdml, pdata);
+				proto_tree_write_node_pdml, pdata);
 		pdata->level--;
 	}
 
@@ -472,7 +391,7 @@ proto_tree_print_node_pdml(proto_node *node, gpointer data)
  * but we produce a 'geninfo' protocol in the PDML to conform to spec.
  * The 'frame' protocol follows the 'geninfo' protocol in the PDML. */
 static void
-print_pdml_geninfo(proto_tree *tree, print_data *pdata)
+print_pdml_geninfo(proto_tree *tree, FILE *fh)
 {
 	guint32 num, len, caplen;
 	nstime_t *timestamp;
@@ -520,37 +439,179 @@ print_pdml_geninfo(proto_tree *tree, print_data *pdata)
 	g_ptr_array_free(finfo_array, FALSE);
 
 	/* Print geninfo start */
-	fprintf(pdata->fh,
+	fprintf(fh,
 "  <proto name=\"geninfo\" pos=\"0\" showname=\"General information\" size=\"%u\">\n",
 		frame_finfo->length);
 
 	/* Print geninfo.num */
-	fprintf(pdata->fh,
+	fprintf(fh,
 "    <field name=\"num\" pos=\"0\" show=\"%u\" showname=\"Number\" value=\"%x\" size=\"%u\"/>\n",
 		num, num, frame_finfo->length);
 
 	/* Print geninfo.len */
-	fprintf(pdata->fh,
+	fprintf(fh,
 "    <field name=\"len\" pos=\"0\" show=\"%u\" showname=\"Packet Length\" value=\"%x\" size=\"%u\"/>\n",
 		len, len, frame_finfo->length);
 
 	/* Print geninfo.caplen */
-	fprintf(pdata->fh,
+	fprintf(fh,
 "    <field name=\"caplen\" pos=\"0\" show=\"%u\" showname=\"Captured Length\" value=\"%x\" size=\"%u\"/>\n",
 		caplen, caplen, frame_finfo->length);
 
 	/* Print geninfo.timestamp */
-	fprintf(pdata->fh,
+	fprintf(fh,
 "    <field name=\"timestamp\" pos=\"0\" show=\"%s\" showname=\"Captured Time\" value=\"%d.%09d\" size=\"%u\"/>\n",
 		abs_time_to_str(timestamp), (int) timestamp->secs, timestamp->nsecs, frame_finfo->length);
 
 	/* Print geninfo end */
-	fprintf(pdata->fh,
+	fprintf(fh,
 "  </proto>\n");
 }
 
+void
+write_pdml_finale(FILE *fh)
+{
+	fputs("</pdml>\n", fh);
+}
 
-void print_hex_data(FILE *fh, print_format_e format, epan_dissect_t *edt)
+void
+write_psml_preamble(FILE *fh)
+{
+	fputs("<?xml version=\"1.0\"?>\n", fh);
+	fputs("<psml version=\"" PSML_VERSION "\" ", fh);
+	fprintf(fh, "creator=\"%s/%s\">\n", PACKAGE, VERSION);
+}
+
+void
+proto_tree_write_psml(epan_dissect_t *edt, FILE *fh)
+{
+	gint	i;
+
+	/* if this is the first packet, we have to create the PSML structure output */
+	if(edt->pi.fd->num == 1) {
+	    fprintf(fh, "<structure>\n");
+
+	    for(i=0; i < edt->pi.cinfo->num_cols; i++) {
+		fprintf(fh, "<section>");
+		print_escaped_xml(fh, edt->pi.cinfo->col_title[i]);
+		fprintf(fh, "</section>\n");
+	    }
+
+	    fprintf(fh, "</structure>\n\n");
+	}
+
+	fprintf(fh, "<packet>\n");
+
+	for(i=0; i < edt->pi.cinfo->num_cols; i++) {
+	    fprintf(fh, "<section>");
+	    print_escaped_xml(fh, edt->pi.cinfo->col_data[i]);
+	    fprintf(fh, "</section>\n");
+	}
+
+	fprintf(fh, "</packet>\n\n");
+}
+
+void
+write_psml_finale(FILE *fh)
+{
+	fputs("</psml>\n", fh);
+}
+
+/*
+ * Find the data source for a specified field, and return a pointer
+ * to the data in it. Returns NULL if the data is out of bounds.
+ */
+static const guint8 *
+get_field_data(GSList *src_list, field_info *fi)
+{
+	GSList *src_le;
+	data_source *src;
+	tvbuff_t *src_tvb;
+	gint length, tvbuff_length;
+
+	for (src_le = src_list; src_le != NULL; src_le = src_le->next) {
+		src = src_le->data;
+		src_tvb = src->tvb;
+		if (fi->ds_tvb == src_tvb) {
+			/*
+			 * Found it.
+			 *
+			 * XXX - a field can have a length that runs past
+			 * the end of the tvbuff.  Ideally, that should
+			 * be fixed when adding an item to the protocol
+			 * tree, but checking the length when doing
+			 * that could be expensive.  Until we fix that,
+			 * we'll do the check here.
+			 */
+			tvbuff_length = tvb_length_remaining(src_tvb,
+			    fi->start);
+			if (tvbuff_length < 0) {
+				return NULL;
+			}
+			length = fi->length;
+			if (length > tvbuff_length)
+				length = tvbuff_length;
+			return tvb_get_ptr(src_tvb, fi->start, length);
+		}
+	}
+	g_assert_not_reached();
+	return NULL;	/* not found */
+}
+
+/* Print a string, escaping out certain characters that need to
+ * escaped out for XML. */
+static void
+print_escaped_xml(FILE *fh, char *unescaped_string)
+{
+	unsigned char *p;
+
+	for (p = unescaped_string; *p != '\0'; p++) {
+		switch (*p) {
+			case '&':
+				fputs("&amp;", fh);
+				break;
+			case '<':
+				fputs("&lt;", fh);
+				break;
+			case '>':
+				fputs("&gt;", fh);
+				break;
+			case '"':
+				fputs("&quot;", fh);
+				break;
+			case '\'':
+				fputs("&apos;", fh);
+				break;
+			default:
+				fputc(*p, fh);
+		}
+	}
+}
+
+static void
+write_pdml_field_hex_value(write_pdml_data *pdata, field_info *fi)
+{
+	int i;
+	const guint8 *pd;
+
+	if (fi->length > tvb_length_remaining(fi->ds_tvb, fi->start)) {
+		fprintf(pdata->fh, "field length invalid!");
+		return;
+	}
+
+	/* Find the data for this field. */
+	pd = get_field_data(pdata->src_list, fi);
+
+	if (pd) {
+		/* Print a simple hex dump */
+		for (i = 0 ; i < fi->length; i++) {
+			fprintf(pdata->fh, "%02x", pd[i]);
+		}
+	}
+}
+
+gboolean
+print_hex_data(print_stream_t *stream, epan_dissect_t *edt)
 {
 	gboolean multiple_sources;
 	GSList *src_le;
@@ -575,18 +636,20 @@ void print_hex_data(FILE *fh, print_format_e format, epan_dissect_t *edt)
 		tvb = src->tvb;
 		if (multiple_sources) {
 			name = src->name;
-			print_line(fh, 0, format, "");
+			print_line(stream, 0, "");
 			line = g_malloc(strlen(name) + 2);	/* <name>:\0 */
 			strcpy(line, name);
 			strcat(line, ":");
-			print_line(fh, 0, format, line);
+			print_line(stream, 0, line);
 			g_free(line);
 		}
 		length = tvb_length(tvb);
 		cp = tvb_get_ptr(tvb, 0, length);
-		print_hex_data_buffer(fh, cp, length,
-		    edt->pi.fd->flags.encoding, format);
+		if (!print_hex_data_buffer(stream, cp, length,
+		    edt->pi.fd->flags.encoding))
+			return FALSE;
 	}
+	return TRUE;
 }
 
 /*
@@ -611,9 +674,9 @@ void print_hex_data(FILE *fh, print_format_e format, epan_dissect_t *edt)
 				   offset, 2 blanks separating offset
 				   from data dump, data dump */
 
-static void
-print_hex_data_buffer(FILE *fh, register const guchar *cp,
-    register guint length, char_enc encoding, print_format_e format)
+static gboolean
+print_hex_data_buffer(print_stream_t *stream, const guchar *cp,
+    guint length, char_enc encoding)
 {
 	register unsigned int ad, i, j, k, l;
 	guchar c;
@@ -623,7 +686,8 @@ print_hex_data_buffer(FILE *fh, register const guchar *cp,
 		'0', '1', '2', '3', '4', '5', '6', '7',
 		'8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
 
-	print_line(fh, 0, format, "");
+	if (!print_line(stream, 0, ""))
+		return FALSE;
 
 	/*
 	 * How many of the leading digits of the offset will we supply?
@@ -684,10 +748,12 @@ print_hex_data_buffer(FILE *fh, register const guchar *cp,
 			 * and advance the offset.
 			 */
 			line[k] = '\0';
-			print_line(fh, 0, format, line);
+			if (!print_line(stream, 0, line))
+				return FALSE;
 			ad += 16;
 		}
 	}
+	return TRUE;
 }
 
 static
@@ -719,147 +785,299 @@ void ps_clean_string(unsigned char *out, const unsigned char *in,
 }
 
 /* Some formats need stuff at the beginning of the output */
-void
-print_preamble(FILE *fh, print_format_e format, gchar *filename)
+gboolean
+print_preamble(print_stream_t *self, gchar *filename)
 {
-	char		psbuffer[MAX_PS_LINE_LENGTH]; /* static sized buffer! */
-
-    
-    switch(format) {
-    case(PR_FMT_TEXT):
-        /* do nothing */
-        break;
-    case(PR_FMT_PS):
-		print_ps_preamble(fh);
-
-		fputs("%% Set the font to 10 point\n", fh);
-		fputs("/Courier findfont 10 scalefont setfont\n", fh);
-		fputs("\n", fh);
-		fputs("%% the page title\n", fh);
-		ps_clean_string(psbuffer, filename, MAX_PS_LINE_LENGTH);
-		fprintf(fh, "/eth_pagetitle (%s - Ethereal) def\n", psbuffer);
-		fputs("\n", fh);
-        break;
-    case(PR_FMT_PDML):
-		fputs("<?xml version=\"1.0\"?>\n", fh);
-		fputs("<pdml version=\"" PDML_VERSION "\" ", fh);
-		fprintf(fh, "creator=\"%s/%s\">\n", PACKAGE, VERSION);
-        break;
-    case(PR_FMT_PSML):
-		fputs("<?xml version=\"1.0\"?>\n", fh);
-		fputs("<psml version=\"" PSML_VERSION "\" ", fh);
-		fprintf(fh, "creator=\"%s/%s\">\n", PACKAGE, VERSION);
-        break;
-    default:
-		g_assert_not_reached();
-	}
+	return (self->ops->print_preamble)(self, filename);
 }
 
-void
-print_packet_header(FILE *fh, print_format_e format, guint32 number, gchar *summary) {
-	char		psbuffer[MAX_PS_LINE_LENGTH]; /* static sized buffer! */
-
-    
-    switch(format) {
-    case(PR_FMT_TEXT):
-        /* do nothing */
-        break;
-    case(PR_FMT_PS):
-		ps_clean_string(psbuffer, summary, MAX_PS_LINE_LENGTH);
-        fprintf(fh, "[/Dest /__frame%u__ /Title (%s)   /OUT pdfmark\n", number, psbuffer);
-        fputs("[/View [/XYZ -4 currentpoint matrix currentmatrix matrix defaultmatrix\n", fh);
-        fputs("matrix invertmatrix matrix concatmatrix transform exch pop 20 add null]\n", fh);
-        fprintf(fh, "/Dest /__frame%u__ /DEST pdfmark\n", number);
-        break;
-    case(PR_FMT_PDML):
-        /* do nothing */
-        break;
-    case(PR_FMT_PSML):
-        /* do nothing */
-        break;
-    default:
-		g_assert_not_reached();
-	}
+gboolean
+print_line(print_stream_t *self, int indent, const char *line)
+{
+	return (self->ops->print_line)(self, indent, line);
 }
 
-void
-print_formfeed(FILE *fh, print_format_e format) {
-    switch(format) {
-    case(PR_FMT_TEXT):
-		fputs("\f", fh);
-        break;
-    case(PR_FMT_PS):
-		fputs("formfeed\n", fh);
-        break;
-    case(PR_FMT_PDML):
-        /* do nothing */
-        break;
-    case(PR_FMT_PSML):
-        /* do nothing */
-        break;
-    default:
-		g_assert_not_reached();
-	}
+/* Insert bookmark */
+gboolean
+print_bookmark(print_stream_t *self, const gchar *name, const gchar *title)
+{
+	return (self->ops->print_bookmark)(self, name, title);
+}
+
+gboolean
+new_page(print_stream_t *self)
+{
+	return (self->ops->new_page)(self);
 }
 
 /* Some formats need stuff at the end of the output */
-void
-print_finale(FILE *fh, print_format_e format)
+gboolean
+print_finale(print_stream_t *self)
 {
-    switch(format) {
-    case(PR_FMT_TEXT):
-        /* do nothing */
-        break;
-    case(PR_FMT_PS):
-		print_ps_finale(fh);
-        break;
-    case(PR_FMT_PDML):
-		fputs("</pdml>\n", fh);
-        break;
-    case(PR_FMT_PSML):
-		fputs("</psml>\n", fh);
-        break;
-    default:
-		g_assert_not_reached();
-	}
+	return (self->ops->print_finale)(self);
 }
 
-void
-print_line(FILE *fh, int indent, print_format_e format, char *line)
+gboolean
+destroy_print_stream(print_stream_t *self)
 {
-	char		space[MAX_INDENT+1];
-	int		i;
-	int		num_spaces;
+	return (self->ops->destroy)(self);
+}
+
+typedef struct {
+	int to_file;
+	FILE *fh;
+} output_text;
+
+static gboolean
+print_preamble_text(print_stream_t *self _U_, gchar *filename _U_)
+{
+	/* do nothing */
+	return TRUE;	/* always succeeds */
+}
+
+static gboolean
+print_line_text(print_stream_t *self, int indent, const char *line)
+{
+	output_text *output = self->data;
+	char space[MAX_INDENT+1];
+	int i;
+	int num_spaces;
+
+	/* Prepare the tabs for printing, depending on tree level */
+	num_spaces = indent * 4;
+	if (num_spaces > MAX_INDENT) {
+		num_spaces = MAX_INDENT;
+	}
+	for (i = 0; i < num_spaces; i++) {
+		space[i] = ' ';
+	}
+	/* The string is NUL-terminated */
+	space[num_spaces] = '\0';
+
+	fputs(space, output->fh);
+	fputs(line, output->fh);
+	putc('\n', output->fh);
+	return !ferror(output->fh);
+}
+
+static gboolean
+print_bookmark_text(print_stream_t *self _U_, const gchar *name _U_,
+    const gchar *title _U_)
+{
+	/* do nothing */
+	return TRUE;
+}
+
+static gboolean
+new_page_text(print_stream_t *self)
+{
+	output_text *output = self->data;
+
+	fputs("\f", output->fh);
+	return !ferror(output->fh);
+}
+
+static gboolean
+print_finale_text(print_stream_t *self _U_)
+{
+	/* do nothing */
+	return TRUE;	/* always succeeds */
+}
+
+static gboolean
+destroy_text(print_stream_t *self)
+{
+	output_text *output = self->data;
+	gboolean ret;
+
+	ret = close_print_dest(output->to_file, output->fh);
+	g_free(output);
+	g_free(self);
+	return ret;
+}
+
+static const print_stream_ops_t print_text_ops = {
+	print_preamble_text,
+	print_line_text,
+	print_bookmark_text,
+	new_page_text,
+	print_finale_text,
+	destroy_text
+};
+
+print_stream_t *
+print_stream_text_new(int to_file, const char *dest)
+{
+	FILE *fh;
+	print_stream_t *stream;
+	output_text *output;
+
+	fh = open_print_dest(to_file, dest);
+	if (fh == NULL)
+		return NULL;
+
+	output = g_malloc(sizeof *output);
+	output->to_file = to_file;
+	output->fh = fh;
+	stream = g_malloc(sizeof (print_stream_t));
+	stream->ops = &print_text_ops;
+	stream->data = output;
+
+	return stream;
+}
+
+print_stream_t *
+print_stream_text_stdio_new(FILE *fh)
+{
+	print_stream_t *stream;
+	output_text *output;
+
+	output = g_malloc(sizeof *output);
+	output->to_file = TRUE;
+	output->fh = fh;
+	stream = g_malloc(sizeof (print_stream_t));
+	stream->ops = &print_text_ops;
+	stream->data = output;
+
+	return stream;
+}
+
+typedef struct {
+	int to_file;
+	FILE *fh;
+} output_ps;
+
+static gboolean
+print_preamble_ps(print_stream_t *self, gchar *filename)
+{
+	output_ps *output = self->data;
 	char		psbuffer[MAX_PS_LINE_LENGTH]; /* static sized buffer! */
 
-    switch(format) {
-    case(PR_FMT_TEXT):
-		/* Prepare the tabs for printing, depending on tree level */
-		num_spaces = indent * 4;
-		if (num_spaces > MAX_INDENT) {
-			num_spaces = MAX_INDENT;
-		}
-		for (i = 0; i < num_spaces; i++) {
-			space[i] = ' ';
-		}
-		/* The string is NUL-terminated */
-		space[num_spaces] = '\0';
+	print_ps_preamble(output->fh);
 
-		fputs(space, fh);
-		fputs(line, fh);
-		putc('\n', fh);
-        break;
-    case(PR_FMT_PS):
-		ps_clean_string(psbuffer, line, MAX_PS_LINE_LENGTH);
-		fprintf(fh, "%d (%s) putline\n", indent, psbuffer);
-        break;
-    case(PR_FMT_PDML):
-        /* do nothing */
-        break;
-    case(PR_FMT_PSML):
-        /* do nothing */
-        break;
-    default:
-		g_assert_not_reached();
-    }
+	fputs("%% Set the font to 10 point\n", output->fh);
+	fputs("/Courier findfont 10 scalefont setfont\n", output->fh);
+	fputs("\n", output->fh);
+	fputs("%% the page title\n", output->fh);
+	ps_clean_string(psbuffer, filename, MAX_PS_LINE_LENGTH);
+	fprintf(output->fh, "/eth_pagetitle (%s - Ethereal) def\n", psbuffer);
+	fputs("\n", output->fh);
+	return !ferror(output->fh);
+}
+
+static gboolean
+print_line_ps(print_stream_t *self, int indent, const char *line)
+{
+	output_ps *output = self->data;
+	char		psbuffer[MAX_PS_LINE_LENGTH]; /* static sized buffer! */
+
+	ps_clean_string(psbuffer, line, MAX_PS_LINE_LENGTH);
+	fprintf(output->fh, "%d (%s) putline\n", indent, psbuffer);
+	return !ferror(output->fh);
+}
+
+static gboolean
+print_bookmark_ps(print_stream_t *self, const gchar *name, const gchar *title)
+{
+	output_ps *output = self->data;
+	char		psbuffer[MAX_PS_LINE_LENGTH]; /* static sized buffer! */
+
+	/*
+	 * See the Adobe "pdfmark reference":
+	 *
+	 *	http://partners.adobe.com/asn/acrobat/docs/pdfmark.pdf
+	 *
+	 * The pdfmark stuff tells code that turns PostScript into PDF
+	 * things that it should do.
+	 *
+	 * The /OUT stuff creates a bookmark that goes to the
+	 * destination with "name" as the name and "title" as the title.
+	 *
+	 * The "/DEST" creates the destination.
+	 */
+	ps_clean_string(psbuffer, title, MAX_PS_LINE_LENGTH);
+	fprintf(output->fh, "[/Dest /%s /Title (%s)   /OUT pdfmark\n", name,
+	    psbuffer);
+	fputs("[/View [/XYZ -4 currentpoint matrix currentmatrix matrix defaultmatrix\n",
+	    output->fh);
+	fputs("matrix invertmatrix matrix concatmatrix transform exch pop 20 add null]\n",
+	    output->fh);
+	fprintf(output->fh, "/Dest /%s /DEST pdfmark\n", name);
+	return !ferror(output->fh);
+}
+
+static gboolean
+new_page_ps(print_stream_t *self)
+{
+	output_ps *output = self->data;
+
+	fputs("formfeed\n", output->fh);
+	return !ferror(output->fh);
+}
+
+static gboolean
+print_finale_ps(print_stream_t *self)
+{
+	output_ps *output = self->data;
+
+	print_ps_finale(output->fh);
+	return !ferror(output->fh);
+}
+
+static gboolean
+destroy_ps(print_stream_t *self)
+{
+	output_ps *output = self->data;
+	gboolean ret;
+
+	ret = close_print_dest(output->to_file, output->fh);
+	g_free(output);
+	g_free(self);
+	return ret;
+}
+
+static const print_stream_ops_t print_ps_ops = {
+	print_preamble_ps,
+	print_line_ps,
+	print_bookmark_ps,
+	new_page_ps,
+	print_finale_ps,
+	destroy_ps
+};
+
+print_stream_t *
+print_stream_ps_new(int to_file, const char *dest)
+{
+	FILE *fh;
+	print_stream_t *stream;
+	output_ps *output;
+
+	fh = open_print_dest(to_file, dest);
+	if (fh == NULL)
+		return NULL;
+
+	output = g_malloc(sizeof *output);
+	output->to_file = to_file;
+	output->fh = fh;
+	stream = g_malloc(sizeof (print_stream_t));
+	stream->ops = &print_ps_ops;
+	stream->data = output;
+
+	return stream;
+}
+
+print_stream_t *
+print_stream_ps_stdio_new(FILE *fh)
+{
+	print_stream_t *stream;
+	output_ps *output;
+
+	output = g_malloc(sizeof *output);
+	output->to_file = TRUE;
+	output->fh = fh;
+	stream = g_malloc(sizeof (print_stream_t));
+	stream->ops = &print_ps_ops;
+	stream->data = output;
+
+	return stream;
 }
