@@ -1,7 +1,7 @@
 /* tap-dcerpcstat.c
  * dcerpcstat   2002 Ronnie Sahlberg
  *
- * $Id: tap-dcerpcstat.c,v 1.9 2004/01/19 23:43:29 guy Exp $
+ * $Id: tap-dcerpcstat.c 12128 2004-09-29 00:06:36Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,8 +34,8 @@
 
 #include <string.h>
 #include "epan/packet_info.h"
-#include "tap.h"
-#include "packet-dcerpc.h"
+#include <epan/tap.h>
+#include <epan/dissectors/packet-dcerpc.h>
 #include "register.h"
 
 /* used to keep track of statistics for a specific procedure */
@@ -80,7 +80,7 @@ dcerpcstat_packet(void *prs, packet_info *pinfo, epan_dissect_t *edt _U_, void *
 	}
 
 	/* we are only interested in reply packets */
-	if(ri->request){
+	if(ri->ptype != PDU_RESP){
 		return 0;
 	}
 
