@@ -1,7 +1,7 @@
 /* tap-iostat.c
  * iostat   2002 Ronnie Sahlberg
  *
- * $Id: tap-iostat.c,v 1.9 2003/12/02 21:15:46 guy Exp $
+ * $Id: tap-iostat.c 12128 2004-09-29 00:06:36Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,7 +35,7 @@
 #include <string.h>
 #include "epan/epan_dissect.h"
 #include "epan/packet_info.h"
-#include "tap.h"
+#include <epan/tap.h>
 #include "register.h"
 
 
