@@ -3,7 +3,7 @@
  * exists so that the library can be loaded on systems that
  * have it.
  *
- * $Id: xmlstub.c,v 1.4 2004/04/25 10:38:24 etxrab Exp $
+ * $Id: xmlstub.c 12341 2004-10-18 15:14:13Z gerald $
  *
  * Copyright (c) 2001 by David Frascone <dave@frascone.com>
  *
@@ -145,11 +145,13 @@ loadLibXML(void)
 	}
 	XmlStub.xmlSubstituteEntitiesDefault=(int(*)(int))symbol;
 
-	if (!g_module_symbol(handle, "xmlDoValidityCheckingDefaultValue", &symbol)) {
+#ifdef ETHEREAL_XML_DO_VALIDITY_CHECKING
+  if (!g_module_symbol(handle, "xmlDoValidityCheckingDefaultValue", &symbol)) {
 		g_warning("Unable to find \"xmlDoValidityCheckingDefaultValue\"");
 		error=TRUE;
 	}
 	XmlStub.xmlDoValidityCheckingDefaultValue = (int *)symbol;
+#endif
 
 	/*
 	 * Return if any of the above functions set our error flag.
